# pylint: disable=line-too-long
"""The auxin commands module.

These commands are used in the following models:
DPO5K, DPO5KB, DPO70KC, DPO70KD, DPO70KDX, DPO70KSX, DPO7K, DPO7KC, DSA70KC, DSA70KD, MSO5K, MSO5KB,
MSO70KC, MSO70KDX

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - AUXIn:BANdwidth <NR3>
    - AUXIn:BANdwidth?
    - AUXIn:COUPling <NR3>
    - AUXIn:COUPling?
    - AUXIn:OFFSet <NR3>
    - AUXIn:OFFSet?
    - AUXIn:PROBEFunc:EXTAtten <NR3>
    - AUXIn:PROBEFunc:EXTAtten?
    - AUXIn:PROBEFunc:EXTDBatten <NR3>
    - AUXIn:PROBEFunc:EXTDBatten?
    - AUXIn:PROBEFunc:EXTUnits <QString>
    - AUXIn:PROBEFunc:EXTUnits?
    - AUXIn:PRObe:AUTOZero EXECute
    - AUXIn:PRObe:DEGAUSS EXECute
    - AUXIn:PRObe:DEGAUSS:STATE?
    - AUXIn:PRObe:FORCEDRange <NR3>
    - AUXIn:PRObe:FORCEDRange?
    - AUXIn:PRObe:GAIN?
    - AUXIn:PRObe:ID:SERnumber?
    - AUXIn:PRObe:ID:TYPe?
    - AUXIn:PRObe:INPUTMode {COMmonmode|DEFault|DIFFerential|A|B}
    - AUXIn:PRObe:INPUTMode:AOFFSet <NR3>
    - AUXIn:PRObe:INPUTMode:AOFFSet?
    - AUXIn:PRObe:INPUTMode:BOFFSet <NR3>
    - AUXIn:PRObe:INPUTMode:BOFFSet?
    - AUXIn:PRObe:INPUTMode:CMOFFSet <NR3>
    - AUXIn:PRObe:INPUTMode:CMOFFSet?
    - AUXIn:PRObe:INPUTMode:DMOFFSet <NR3>
    - AUXIn:PRObe:INPUTMode:DMOFFSet?
    - AUXIn:PRObe:INPUTMode?
    - AUXIn:PRObe:RANge {ATTEN1X|ATTEN1_25X|ATTEN1_5X|ATTEN1_75X|ATTEN2X|ATTEN2_5X|ATTEN3X|ATTEN3_5X|ATTEN4X|ATTEN4_5X|ATTEN5X|ATTEN5_5X|ATTEN6X|ATTEN6_5X|ATTEN7X|ATTEN7_5X|ATTEN8X|ATTEN8_5X|ATTEN9X|ATTEN9_5X|ATTEN10X|ATTEN12_5X|ATTEN15X|ATTEN17_5X|ATTEN20X|ATTEN25X|ATTEN30X|ATTEN35X|ATTEN40X|ATTEN45X|ATTEN50X|ATTEN55X|ATTEN60X|ATTEN65X|ATTEN70X|ATTEN75X|ATTEN80X|ATTEN85X|ATTEN90X|ATTEN95X|ATTEN100X|ATTEN125X|ATTEN150X|ATTEN175X|ATTEN200X|ATTEN250X|ATTEN300X|ATTEN350X|ATTEN400X|ATTEN450X|ATTEN500X|ATTEN550X|ATTEN600X|ATTEN650X|ATTEN700X|ATTEN750X|ATTEN800X|ATTEN850X|ATTEN900X|ATTEN950X|ATTEN1000X}
    - AUXIn:PRObe:RANge?
    - AUXIn:PRObe:RESistance?
    - AUXIn:PRObe:SET <QString>
    - AUXIn:PRObe:SET?
    - AUXIn:PRObe:SIGnal {PASS|BYPass}
    - AUXIn:PRObe:SIGnal?
    - AUXIn:PRObe:TIPtype {HBWStraightflex|OTHer}
    - AUXIn:PRObe:TIPtype?
    - AUXIn:PRObe:UNIts?
    - AUXIn:VTERm:DUAL:A <NR3>
    - AUXIn:VTERm:DUAL:A?
    - AUXIn:VTERm:DUAL:B <NR3>
    - AUXIn:VTERm:DUAL:B?
    ```
"""  # noqa: E501

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class AuxinVtermDualB(SCPICmdWrite, SCPICmdRead):
    """The ``AUXIn:VTERm:DUAL:B`` command.

    Description:
        - This command sets or queries the termination voltage for probes with dual inputs that
          support settable termination voltage.

    Usage:
        - Using the ``.query()`` method will send the ``AUXIn:VTERm:DUAL:B?`` query.
        - Using the ``.verify(value)`` method will send the ``AUXIn:VTERm:DUAL:B?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AUXIn:VTERm:DUAL:B value`` command.

    SCPI Syntax:
        ```
        - AUXIn:VTERm:DUAL:B <NR3>
        - AUXIn:VTERm:DUAL:B?
        ```

    Info:
        - ``<NR3>`` specifies the termination voltage.
    """


class AuxinVtermDualA(SCPICmdWrite, SCPICmdRead):
    """The ``AUXIn:VTERm:DUAL:A`` command.

    Description:
        - This command sets or queries the termination voltage for probes with dual inputs that
          support settable termination voltage.

    Usage:
        - Using the ``.query()`` method will send the ``AUXIn:VTERm:DUAL:A?`` query.
        - Using the ``.verify(value)`` method will send the ``AUXIn:VTERm:DUAL:A?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AUXIn:VTERm:DUAL:A value`` command.

    SCPI Syntax:
        ```
        - AUXIn:VTERm:DUAL:A <NR3>
        - AUXIn:VTERm:DUAL:A?
        ```

    Info:
        - ``<NR3>`` specifies the termination voltage.
    """


class AuxinVtermDual(SCPICmdRead):
    """The ``AUXIn:VTERm:DUAL`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``AUXIn:VTERm:DUAL?`` query.
        - Using the ``.verify(value)`` method will send the ``AUXIn:VTERm:DUAL?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.a``: The ``AUXIn:VTERm:DUAL:A`` command.
        - ``.b``: The ``AUXIn:VTERm:DUAL:B`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._a = AuxinVtermDualA(device, f"{self._cmd_syntax}:A")
        self._b = AuxinVtermDualB(device, f"{self._cmd_syntax}:B")

    @property
    def a(self) -> AuxinVtermDualA:
        """Return the ``AUXIn:VTERm:DUAL:A`` command.

        Description:
            - This command sets or queries the termination voltage for probes with dual inputs that
              support settable termination voltage.

        Usage:
            - Using the ``.query()`` method will send the ``AUXIn:VTERm:DUAL:A?`` query.
            - Using the ``.verify(value)`` method will send the ``AUXIn:VTERm:DUAL:A?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AUXIn:VTERm:DUAL:A value`` command.

        SCPI Syntax:
            ```
            - AUXIn:VTERm:DUAL:A <NR3>
            - AUXIn:VTERm:DUAL:A?
            ```

        Info:
            - ``<NR3>`` specifies the termination voltage.
        """
        return self._a

    @property
    def b(self) -> AuxinVtermDualB:
        """Return the ``AUXIn:VTERm:DUAL:B`` command.

        Description:
            - This command sets or queries the termination voltage for probes with dual inputs that
              support settable termination voltage.

        Usage:
            - Using the ``.query()`` method will send the ``AUXIn:VTERm:DUAL:B?`` query.
            - Using the ``.verify(value)`` method will send the ``AUXIn:VTERm:DUAL:B?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AUXIn:VTERm:DUAL:B value`` command.

        SCPI Syntax:
            ```
            - AUXIn:VTERm:DUAL:B <NR3>
            - AUXIn:VTERm:DUAL:B?
            ```

        Info:
            - ``<NR3>`` specifies the termination voltage.
        """
        return self._b


class AuxinVterm(SCPICmdRead):
    """The ``AUXIn:VTERm`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``AUXIn:VTERm?`` query.
        - Using the ``.verify(value)`` method will send the ``AUXIn:VTERm?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.dual``: The ``AUXIn:VTERm:DUAL`` command tree.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._dual = AuxinVtermDual(device, f"{self._cmd_syntax}:DUAL")

    @property
    def dual(self) -> AuxinVtermDual:
        """Return the ``AUXIn:VTERm:DUAL`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``AUXIn:VTERm:DUAL?`` query.
            - Using the ``.verify(value)`` method will send the ``AUXIn:VTERm:DUAL?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.a``: The ``AUXIn:VTERm:DUAL:A`` command.
            - ``.b``: The ``AUXIn:VTERm:DUAL:B`` command.
        """
        return self._dual


class AuxinProbeUnits(SCPICmdRead):
    """The ``AUXIn:PRObe:UNIts`` command.

    Description:
        - This query-only command returns a string describing the units of measure for the probe
          attached to the AUX In input.

    Usage:
        - Using the ``.query()`` method will send the ``AUXIn:PRObe:UNIts?`` query.
        - Using the ``.verify(value)`` method will send the ``AUXIn:PRObe:UNIts?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - AUXIn:PRObe:UNIts?
        ```
    """


class AuxinProbeTiptype(SCPICmdWrite, SCPICmdRead):
    """The ``AUXIn:PRObe:TIPtype`` command.

    Description:
        - This command sets or queries the type of probe tip being used.

    Usage:
        - Using the ``.query()`` method will send the ``AUXIn:PRObe:TIPtype?`` query.
        - Using the ``.verify(value)`` method will send the ``AUXIn:PRObe:TIPtype?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AUXIn:PRObe:TIPtype value`` command.

    SCPI Syntax:
        ```
        - AUXIn:PRObe:TIPtype {HBWStraightflex|OTHer}
        - AUXIn:PRObe:TIPtype?
        ```

    Info:
        - ``HBWStraightflex`` lets the instrument know you are using a high bandwidth straight-flex
          probe tip.
        - ``OTHer`` lets the instrument know you are not using a high bandwidth straight-flex probe
          tip.
    """


class AuxinProbeSignal(SCPICmdWrite, SCPICmdRead):
    """The ``AUXIn:PRObe:SIGnal`` command.

    Description:
        - This command sets or queries aspects of probe accessory user interfaces. The available
          arguments for this command will vary depending on the accessory you attach to the
          instrument.

    Usage:
        - Using the ``.query()`` method will send the ``AUXIn:PRObe:SIGnal?`` query.
        - Using the ``.verify(value)`` method will send the ``AUXIn:PRObe:SIGnal?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AUXIn:PRObe:SIGnal value`` command.

    SCPI Syntax:
        ```
        - AUXIn:PRObe:SIGnal {PASS|BYPass}
        - AUXIn:PRObe:SIGnal?
        ```

    Info:
        - ``PASS`` opens a relay passing your signal to the instrument.
        - ``BYPass`` closes a relay preventing your signal from reaching the instrument. During
          probe degauss, the signal should be bypassed.
    """


class AuxinProbeSet(SCPICmdWrite, SCPICmdRead):
    """The ``AUXIn:PRObe:SET`` command.

    Description:
        - This command sets or queries aspects of probe accessory user interfaces, for example probe
          attenuation factors. The available arguments for this command will vary depending on the
          accessory you attach to the instrument auxiliary trigger input. For the P7260 probe, you
          can select between two attenuation factors using either this GPIB command or the push
          switch on the probe. The probe enables the relevant path and adjusts the settings based on
          the characteristics of the path in use. The probe signal path selection is not kept in
          persistent storage. The probe will lose the selection if you reboot the instrument or
          remove the probe. Also, the instrument does not store the selection in the save/recall
          setup operation.

    Usage:
        - Using the ``.query()`` method will send the ``AUXIn:PRObe:SET?`` query.
        - Using the ``.verify(value)`` method will send the ``AUXIn:PRObe:SET?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AUXIn:PRObe:SET value`` command.

    SCPI Syntax:
        ```
        - AUXIn:PRObe:SET <QString>
        - AUXIn:PRObe:SET?
        ```

    Info:
        - ``<QString>`` is a quoted string representing a settable aspect of the attached accessory.
        - ``ATTENUATION 5X`` sets the P7260 probe to ±0.75 V dynamic range with 6 GHz bandwidth and
          5X attenuation.
        - ``ATTENUATION 25X`` sets the P7260 probe to ±3 V dynamic range with 6 GHz bandwidth and
          25X attenuation.
        - ``VTERMsource AUTO`` sets the P7380SMA probe voltage termination source to auto.
        - ``VTERMsource INTernal`` sets the P7380SMA probe voltage termination source to internal.
        - ``VTERMsource EXTernal`` sets the P7380SMA probe voltage termination source to external.
    """

    _WRAP_ARG_WITH_QUOTES = True


class AuxinProbeResistance(SCPICmdRead):
    """The ``AUXIn:PRObe:RESistance`` command.

    Description:
        - This query-only command returns the resistance factor of the probe that is attached to the
          AUX In input.

    Usage:
        - Using the ``.query()`` method will send the ``AUXIn:PRObe:RESistance?`` query.
        - Using the ``.verify(value)`` method will send the ``AUXIn:PRObe:RESistance?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - AUXIn:PRObe:RESistance?
        ```
    """


class AuxinProbeRange(SCPICmdWrite, SCPICmdRead):
    """The ``AUXIn:PRObe:RANge`` command.

    Description:
        - This command controls or queries the set attenuation range of the probe on the AUX In
          input.

    Usage:
        - Using the ``.query()`` method will send the ``AUXIn:PRObe:RANge?`` query.
        - Using the ``.verify(value)`` method will send the ``AUXIn:PRObe:RANge?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AUXIn:PRObe:RANge value`` command.

    SCPI Syntax:
        ```
        - AUXIn:PRObe:RANge {ATTEN1X|ATTEN1_25X|ATTEN1_5X|ATTEN1_75X|ATTEN2X|ATTEN2_5X|ATTEN3X|ATTEN3_5X|ATTEN4X|ATTEN4_5X|ATTEN5X|ATTEN5_5X|ATTEN6X|ATTEN6_5X|ATTEN7X|ATTEN7_5X|ATTEN8X|ATTEN8_5X|ATTEN9X|ATTEN9_5X|ATTEN10X|ATTEN12_5X|ATTEN15X|ATTEN17_5X|ATTEN20X|ATTEN25X|ATTEN30X|ATTEN35X|ATTEN40X|ATTEN45X|ATTEN50X|ATTEN55X|ATTEN60X|ATTEN65X|ATTEN70X|ATTEN75X|ATTEN80X|ATTEN85X|ATTEN90X|ATTEN95X|ATTEN100X|ATTEN125X|ATTEN150X|ATTEN175X|ATTEN200X|ATTEN250X|ATTEN300X|ATTEN350X|ATTEN400X|ATTEN450X|ATTEN500X|ATTEN550X|ATTEN600X|ATTEN650X|ATTEN700X|ATTEN750X|ATTEN800X|ATTEN850X|ATTEN900X|ATTEN950X|ATTEN1000X}
        - AUXIn:PRObe:RANge?
        ```
    """  # noqa: E501


class AuxinProbeInputmodeDmoffset(SCPICmdWrite, SCPICmdRead):
    """The ``AUXIn:PRObe:INPUTMode:DMOFFSet`` command.

    Description:
        - This command sets or queries the requested differential mode offset control of the probe
          that is attached to the AUX In input.

    Usage:
        - Using the ``.query()`` method will send the ``AUXIn:PRObe:INPUTMode:DMOFFSet?`` query.
        - Using the ``.verify(value)`` method will send the ``AUXIn:PRObe:INPUTMode:DMOFFSet?``
          query and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AUXIn:PRObe:INPUTMode:DMOFFSet value``
          command.

    SCPI Syntax:
        ```
        - AUXIn:PRObe:INPUTMode:DMOFFSet <NR3>
        - AUXIn:PRObe:INPUTMode:DMOFFSet?
        ```

    Info:
        - ``<NR3>`` specifies the differential mode offset control.
    """


class AuxinProbeInputmodeCmoffset(SCPICmdWrite, SCPICmdRead):
    """The ``AUXIn:PRObe:INPUTMode:CMOFFSet`` command.

    Description:
        - This command sets or queries the requested common mode offset control of the probe that is
          attached to the AUX In input.

    Usage:
        - Using the ``.query()`` method will send the ``AUXIn:PRObe:INPUTMode:CMOFFSet?`` query.
        - Using the ``.verify(value)`` method will send the ``AUXIn:PRObe:INPUTMode:CMOFFSet?``
          query and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AUXIn:PRObe:INPUTMode:CMOFFSet value``
          command.

    SCPI Syntax:
        ```
        - AUXIn:PRObe:INPUTMode:CMOFFSet <NR3>
        - AUXIn:PRObe:INPUTMode:CMOFFSet?
        ```

    Info:
        - ``<NR3>`` specifies the common mode offset control.
    """


class AuxinProbeInputmodeBoffset(SCPICmdWrite, SCPICmdRead):
    """The ``AUXIn:PRObe:INPUTMode:BOFFSet`` command.

    Description:
        - This command sets or queries the requested B mode offset control of the probe that is
          attached to the AUX In input.

    Usage:
        - Using the ``.query()`` method will send the ``AUXIn:PRObe:INPUTMode:BOFFSet?`` query.
        - Using the ``.verify(value)`` method will send the ``AUXIn:PRObe:INPUTMode:BOFFSet?`` query
          and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AUXIn:PRObe:INPUTMode:BOFFSet value``
          command.

    SCPI Syntax:
        ```
        - AUXIn:PRObe:INPUTMode:BOFFSet <NR3>
        - AUXIn:PRObe:INPUTMode:BOFFSet?
        ```

    Info:
        - ``<NR3>`` specifies the B mode offset control.
    """


class AuxinProbeInputmodeAoffset(SCPICmdWrite, SCPICmdRead):
    """The ``AUXIn:PRObe:INPUTMode:AOFFSet`` command.

    Description:
        - This command sets or queries the requested A mode offset control of the probe that is
          attached to the AUX In input.

    Usage:
        - Using the ``.query()`` method will send the ``AUXIn:PRObe:INPUTMode:AOFFSet?`` query.
        - Using the ``.verify(value)`` method will send the ``AUXIn:PRObe:INPUTMode:AOFFSet?`` query
          and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AUXIn:PRObe:INPUTMode:AOFFSet value``
          command.

    SCPI Syntax:
        ```
        - AUXIn:PRObe:INPUTMode:AOFFSet <NR3>
        - AUXIn:PRObe:INPUTMode:AOFFSet?
        ```

    Info:
        - ``<NR3>`` specifies the A mode offset control.
    """


class AuxinProbeInputmode(SCPICmdWrite, SCPICmdRead):
    """The ``AUXIn:PRObe:INPUTMode`` command.

    Description:
        - This command sets or queries one of the probe's four input modes on the AUX In input.

    Usage:
        - Using the ``.query()`` method will send the ``AUXIn:PRObe:INPUTMode?`` query.
        - Using the ``.verify(value)`` method will send the ``AUXIn:PRObe:INPUTMode?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AUXIn:PRObe:INPUTMode value`` command.

    SCPI Syntax:
        ```
        - AUXIn:PRObe:INPUTMode {COMmonmode|DEFault|DIFFerential|A|B}
        - AUXIn:PRObe:INPUTMode?
        ```

    Info:
        - ``COMmonmode`` sets the probe to route common-mode signals to the host.
        - ``DEFault`` sets the probe to the default mode.
        - ``DIFFerential`` sets the probe to route differential signals to the host.
        - ``A`` sets the probe to route single-ended A signals to the host.
        - ``B`` sets the probe to route single-ended B signals to the host.

    Properties:
        - ``.aoffset``: The ``AUXIn:PRObe:INPUTMode:AOFFSet`` command.
        - ``.boffset``: The ``AUXIn:PRObe:INPUTMode:BOFFSet`` command.
        - ``.cmoffset``: The ``AUXIn:PRObe:INPUTMode:CMOFFSet`` command.
        - ``.dmoffset``: The ``AUXIn:PRObe:INPUTMode:DMOFFSet`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._aoffset = AuxinProbeInputmodeAoffset(device, f"{self._cmd_syntax}:AOFFSet")
        self._boffset = AuxinProbeInputmodeBoffset(device, f"{self._cmd_syntax}:BOFFSet")
        self._cmoffset = AuxinProbeInputmodeCmoffset(device, f"{self._cmd_syntax}:CMOFFSet")
        self._dmoffset = AuxinProbeInputmodeDmoffset(device, f"{self._cmd_syntax}:DMOFFSet")

    @property
    def aoffset(self) -> AuxinProbeInputmodeAoffset:
        """Return the ``AUXIn:PRObe:INPUTMode:AOFFSet`` command.

        Description:
            - This command sets or queries the requested A mode offset control of the probe that is
              attached to the AUX In input.

        Usage:
            - Using the ``.query()`` method will send the ``AUXIn:PRObe:INPUTMode:AOFFSet?`` query.
            - Using the ``.verify(value)`` method will send the ``AUXIn:PRObe:INPUTMode:AOFFSet?``
              query and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``AUXIn:PRObe:INPUTMode:AOFFSet value`` command.

        SCPI Syntax:
            ```
            - AUXIn:PRObe:INPUTMode:AOFFSet <NR3>
            - AUXIn:PRObe:INPUTMode:AOFFSet?
            ```

        Info:
            - ``<NR3>`` specifies the A mode offset control.
        """
        return self._aoffset

    @property
    def boffset(self) -> AuxinProbeInputmodeBoffset:
        """Return the ``AUXIn:PRObe:INPUTMode:BOFFSet`` command.

        Description:
            - This command sets or queries the requested B mode offset control of the probe that is
              attached to the AUX In input.

        Usage:
            - Using the ``.query()`` method will send the ``AUXIn:PRObe:INPUTMode:BOFFSet?`` query.
            - Using the ``.verify(value)`` method will send the ``AUXIn:PRObe:INPUTMode:BOFFSet?``
              query and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``AUXIn:PRObe:INPUTMode:BOFFSet value`` command.

        SCPI Syntax:
            ```
            - AUXIn:PRObe:INPUTMode:BOFFSet <NR3>
            - AUXIn:PRObe:INPUTMode:BOFFSet?
            ```

        Info:
            - ``<NR3>`` specifies the B mode offset control.
        """
        return self._boffset

    @property
    def cmoffset(self) -> AuxinProbeInputmodeCmoffset:
        """Return the ``AUXIn:PRObe:INPUTMode:CMOFFSet`` command.

        Description:
            - This command sets or queries the requested common mode offset control of the probe
              that is attached to the AUX In input.

        Usage:
            - Using the ``.query()`` method will send the ``AUXIn:PRObe:INPUTMode:CMOFFSet?`` query.
            - Using the ``.verify(value)`` method will send the ``AUXIn:PRObe:INPUTMode:CMOFFSet?``
              query and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``AUXIn:PRObe:INPUTMode:CMOFFSet value`` command.

        SCPI Syntax:
            ```
            - AUXIn:PRObe:INPUTMode:CMOFFSet <NR3>
            - AUXIn:PRObe:INPUTMode:CMOFFSet?
            ```

        Info:
            - ``<NR3>`` specifies the common mode offset control.
        """
        return self._cmoffset

    @property
    def dmoffset(self) -> AuxinProbeInputmodeDmoffset:
        """Return the ``AUXIn:PRObe:INPUTMode:DMOFFSet`` command.

        Description:
            - This command sets or queries the requested differential mode offset control of the
              probe that is attached to the AUX In input.

        Usage:
            - Using the ``.query()`` method will send the ``AUXIn:PRObe:INPUTMode:DMOFFSet?`` query.
            - Using the ``.verify(value)`` method will send the ``AUXIn:PRObe:INPUTMode:DMOFFSet?``
              query and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``AUXIn:PRObe:INPUTMode:DMOFFSet value`` command.

        SCPI Syntax:
            ```
            - AUXIn:PRObe:INPUTMode:DMOFFSet <NR3>
            - AUXIn:PRObe:INPUTMode:DMOFFSet?
            ```

        Info:
            - ``<NR3>`` specifies the differential mode offset control.
        """
        return self._dmoffset


class AuxinProbeIdType(SCPICmdRead):
    """The ``AUXIn:PRObe:ID:TYPe`` command.

    Description:
        - This query-only command returns the probe type.

    Usage:
        - Using the ``.query()`` method will send the ``AUXIn:PRObe:ID:TYPe?`` query.
        - Using the ``.verify(value)`` method will send the ``AUXIn:PRObe:ID:TYPe?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - AUXIn:PRObe:ID:TYPe?
        ```
    """


class AuxinProbeIdSernumber(SCPICmdRead):
    """The ``AUXIn:PRObe:ID:SERnumber`` command.

    Description:
        - This query-only command returns the probe serial number.

    Usage:
        - Using the ``.query()`` method will send the ``AUXIn:PRObe:ID:SERnumber?`` query.
        - Using the ``.verify(value)`` method will send the ``AUXIn:PRObe:ID:SERnumber?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - AUXIn:PRObe:ID:SERnumber?
        ```
    """


class AuxinProbeId(SCPICmdRead):
    """The ``AUXIn:PRObe:ID`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``AUXIn:PRObe:ID?`` query.
        - Using the ``.verify(value)`` method will send the ``AUXIn:PRObe:ID?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.sernumber``: The ``AUXIn:PRObe:ID:SERnumber`` command.
        - ``.type``: The ``AUXIn:PRObe:ID:TYPe`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._sernumber = AuxinProbeIdSernumber(device, f"{self._cmd_syntax}:SERnumber")
        self._type = AuxinProbeIdType(device, f"{self._cmd_syntax}:TYPe")

    @property
    def sernumber(self) -> AuxinProbeIdSernumber:
        """Return the ``AUXIn:PRObe:ID:SERnumber`` command.

        Description:
            - This query-only command returns the probe serial number.

        Usage:
            - Using the ``.query()`` method will send the ``AUXIn:PRObe:ID:SERnumber?`` query.
            - Using the ``.verify(value)`` method will send the ``AUXIn:PRObe:ID:SERnumber?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - AUXIn:PRObe:ID:SERnumber?
            ```
        """
        return self._sernumber

    @property
    def type(self) -> AuxinProbeIdType:
        """Return the ``AUXIn:PRObe:ID:TYPe`` command.

        Description:
            - This query-only command returns the probe type.

        Usage:
            - Using the ``.query()`` method will send the ``AUXIn:PRObe:ID:TYPe?`` query.
            - Using the ``.verify(value)`` method will send the ``AUXIn:PRObe:ID:TYPe?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - AUXIn:PRObe:ID:TYPe?
            ```
        """
        return self._type


class AuxinProbeGain(SCPICmdRead):
    """The ``AUXIn:PRObe:GAIN`` command.

    Description:
        - This query-only command returns the gain factor of the probe that is attached to the AUX
          In input. The 'gain' of a probe is the output divided by the input transfer ratio. For
          example, a common 10x probe has a gain of 0.1.

    Usage:
        - Using the ``.query()`` method will send the ``AUXIn:PRObe:GAIN?`` query.
        - Using the ``.verify(value)`` method will send the ``AUXIn:PRObe:GAIN?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - AUXIn:PRObe:GAIN?
        ```
    """


class AuxinProbeForcedrange(SCPICmdWrite, SCPICmdRead):
    """The ``AUXIn:PRObe:FORCEDRange`` command.

    Description:
        - This command sets the probe attached to the AUX In input to the specified range, or the
          command queries the probe range.

    Usage:
        - Using the ``.query()`` method will send the ``AUXIn:PRObe:FORCEDRange?`` query.
        - Using the ``.verify(value)`` method will send the ``AUXIn:PRObe:FORCEDRange?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AUXIn:PRObe:FORCEDRange value``
          command.

    SCPI Syntax:
        ```
        - AUXIn:PRObe:FORCEDRange <NR3>
        - AUXIn:PRObe:FORCEDRange?
        ```

    Info:
        - ``<NR3>`` specifies the probe range.
    """


class AuxinProbeDegaussState(SCPICmdRead):
    """The ``AUXIn:PRObe:DEGAUSS:STATE`` command.

    Description:
        - This command queries whether the probe attached to the AUX In input is degaussed.

    Usage:
        - Using the ``.query()`` method will send the ``AUXIn:PRObe:DEGAUSS:STATE?`` query.
        - Using the ``.verify(value)`` method will send the ``AUXIn:PRObe:DEGAUSS:STATE?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - AUXIn:PRObe:DEGAUSS:STATE?
        ```
    """


class AuxinProbeDegauss(SCPICmdWrite, SCPICmdRead):
    """The ``AUXIn:PRObe:DEGAUSS`` command.

    Description:
        - This command starts a degauss cycle of the probe attached to the AUX In input. The degauss
          cycle will change with an appropriate probe attached.

    Usage:
        - Using the ``.write(value)`` method will send the ``AUXIn:PRObe:DEGAUSS value`` command.

    SCPI Syntax:
        ```
        - AUXIn:PRObe:DEGAUSS EXECute
        ```

    Info:
        - ``EXECute`` starts a probe degauss cycle.

    Properties:
        - ``.state``: The ``AUXIn:PRObe:DEGAUSS:STATE`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._state = AuxinProbeDegaussState(device, f"{self._cmd_syntax}:STATE")

    @property
    def state(self) -> AuxinProbeDegaussState:
        """Return the ``AUXIn:PRObe:DEGAUSS:STATE`` command.

        Description:
            - This command queries whether the probe attached to the AUX In input is degaussed.

        Usage:
            - Using the ``.query()`` method will send the ``AUXIn:PRObe:DEGAUSS:STATE?`` query.
            - Using the ``.verify(value)`` method will send the ``AUXIn:PRObe:DEGAUSS:STATE?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - AUXIn:PRObe:DEGAUSS:STATE?
            ```
        """
        return self._state


class AuxinProbeAutozero(SCPICmdWrite):
    """The ``AUXIn:PRObe:AUTOZero`` command.

    Description:
        - The command sets the probe attached to the AUX In input to autozero.

    Usage:
        - Using the ``.write(value)`` method will send the ``AUXIn:PRObe:AUTOZero value`` command.

    SCPI Syntax:
        ```
        - AUXIn:PRObe:AUTOZero EXECute
        ```

    Info:
        - ``EXECute`` sets the probe to autozero.
    """


#  pylint: disable=too-many-instance-attributes
class AuxinProbe(SCPICmdRead):
    """The ``AUXIn:PRObe`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``AUXIn:PRObe?`` query.
        - Using the ``.verify(value)`` method will send the ``AUXIn:PRObe?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.autozero``: The ``AUXIn:PRObe:AUTOZero`` command.
        - ``.degauss``: The ``AUXIn:PRObe:DEGAUSS`` command.
        - ``.forcedrange``: The ``AUXIn:PRObe:FORCEDRange`` command.
        - ``.gain``: The ``AUXIn:PRObe:GAIN`` command.
        - ``.id``: The ``AUXIn:PRObe:ID`` command tree.
        - ``.inputmode``: The ``AUXIn:PRObe:INPUTMode`` command.
        - ``.range``: The ``AUXIn:PRObe:RANge`` command.
        - ``.resistance``: The ``AUXIn:PRObe:RESistance`` command.
        - ``.set``: The ``AUXIn:PRObe:SET`` command.
        - ``.signal``: The ``AUXIn:PRObe:SIGnal`` command.
        - ``.tiptype``: The ``AUXIn:PRObe:TIPtype`` command.
        - ``.units``: The ``AUXIn:PRObe:UNIts`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._autozero = AuxinProbeAutozero(device, f"{self._cmd_syntax}:AUTOZero")
        self._degauss = AuxinProbeDegauss(device, f"{self._cmd_syntax}:DEGAUSS")
        self._forcedrange = AuxinProbeForcedrange(device, f"{self._cmd_syntax}:FORCEDRange")
        self._gain = AuxinProbeGain(device, f"{self._cmd_syntax}:GAIN")
        self._id = AuxinProbeId(device, f"{self._cmd_syntax}:ID")
        self._inputmode = AuxinProbeInputmode(device, f"{self._cmd_syntax}:INPUTMode")
        self._range = AuxinProbeRange(device, f"{self._cmd_syntax}:RANge")
        self._resistance = AuxinProbeResistance(device, f"{self._cmd_syntax}:RESistance")
        self._set = AuxinProbeSet(device, f"{self._cmd_syntax}:SET")
        self._signal = AuxinProbeSignal(device, f"{self._cmd_syntax}:SIGnal")
        self._tiptype = AuxinProbeTiptype(device, f"{self._cmd_syntax}:TIPtype")
        self._units = AuxinProbeUnits(device, f"{self._cmd_syntax}:UNIts")

    @property
    def autozero(self) -> AuxinProbeAutozero:
        """Return the ``AUXIn:PRObe:AUTOZero`` command.

        Description:
            - The command sets the probe attached to the AUX In input to autozero.

        Usage:
            - Using the ``.write(value)`` method will send the ``AUXIn:PRObe:AUTOZero value``
              command.

        SCPI Syntax:
            ```
            - AUXIn:PRObe:AUTOZero EXECute
            ```

        Info:
            - ``EXECute`` sets the probe to autozero.
        """
        return self._autozero

    @property
    def degauss(self) -> AuxinProbeDegauss:
        """Return the ``AUXIn:PRObe:DEGAUSS`` command.

        Description:
            - This command starts a degauss cycle of the probe attached to the AUX In input. The
              degauss cycle will change with an appropriate probe attached.

        Usage:
            - Using the ``.write(value)`` method will send the ``AUXIn:PRObe:DEGAUSS value``
              command.

        SCPI Syntax:
            ```
            - AUXIn:PRObe:DEGAUSS EXECute
            ```

        Info:
            - ``EXECute`` starts a probe degauss cycle.

        Sub-properties:
            - ``.state``: The ``AUXIn:PRObe:DEGAUSS:STATE`` command.
        """
        return self._degauss

    @property
    def forcedrange(self) -> AuxinProbeForcedrange:
        """Return the ``AUXIn:PRObe:FORCEDRange`` command.

        Description:
            - This command sets the probe attached to the AUX In input to the specified range, or
              the command queries the probe range.

        Usage:
            - Using the ``.query()`` method will send the ``AUXIn:PRObe:FORCEDRange?`` query.
            - Using the ``.verify(value)`` method will send the ``AUXIn:PRObe:FORCEDRange?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AUXIn:PRObe:FORCEDRange value``
              command.

        SCPI Syntax:
            ```
            - AUXIn:PRObe:FORCEDRange <NR3>
            - AUXIn:PRObe:FORCEDRange?
            ```

        Info:
            - ``<NR3>`` specifies the probe range.
        """
        return self._forcedrange

    @property
    def gain(self) -> AuxinProbeGain:
        """Return the ``AUXIn:PRObe:GAIN`` command.

        Description:
            - This query-only command returns the gain factor of the probe that is attached to the
              AUX In input. The 'gain' of a probe is the output divided by the input transfer ratio.
              For example, a common 10x probe has a gain of 0.1.

        Usage:
            - Using the ``.query()`` method will send the ``AUXIn:PRObe:GAIN?`` query.
            - Using the ``.verify(value)`` method will send the ``AUXIn:PRObe:GAIN?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - AUXIn:PRObe:GAIN?
            ```
        """
        return self._gain

    @property
    def id(self) -> AuxinProbeId:
        """Return the ``AUXIn:PRObe:ID`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``AUXIn:PRObe:ID?`` query.
            - Using the ``.verify(value)`` method will send the ``AUXIn:PRObe:ID?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.sernumber``: The ``AUXIn:PRObe:ID:SERnumber`` command.
            - ``.type``: The ``AUXIn:PRObe:ID:TYPe`` command.
        """
        return self._id

    @property
    def inputmode(self) -> AuxinProbeInputmode:
        """Return the ``AUXIn:PRObe:INPUTMode`` command.

        Description:
            - This command sets or queries one of the probe's four input modes on the AUX In input.

        Usage:
            - Using the ``.query()`` method will send the ``AUXIn:PRObe:INPUTMode?`` query.
            - Using the ``.verify(value)`` method will send the ``AUXIn:PRObe:INPUTMode?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AUXIn:PRObe:INPUTMode value``
              command.

        SCPI Syntax:
            ```
            - AUXIn:PRObe:INPUTMode {COMmonmode|DEFault|DIFFerential|A|B}
            - AUXIn:PRObe:INPUTMode?
            ```

        Info:
            - ``COMmonmode`` sets the probe to route common-mode signals to the host.
            - ``DEFault`` sets the probe to the default mode.
            - ``DIFFerential`` sets the probe to route differential signals to the host.
            - ``A`` sets the probe to route single-ended A signals to the host.
            - ``B`` sets the probe to route single-ended B signals to the host.

        Sub-properties:
            - ``.aoffset``: The ``AUXIn:PRObe:INPUTMode:AOFFSet`` command.
            - ``.boffset``: The ``AUXIn:PRObe:INPUTMode:BOFFSet`` command.
            - ``.cmoffset``: The ``AUXIn:PRObe:INPUTMode:CMOFFSet`` command.
            - ``.dmoffset``: The ``AUXIn:PRObe:INPUTMode:DMOFFSet`` command.
        """
        return self._inputmode

    @property
    def range(self) -> AuxinProbeRange:
        """Return the ``AUXIn:PRObe:RANge`` command.

        Description:
            - This command controls or queries the set attenuation range of the probe on the AUX In
              input.

        Usage:
            - Using the ``.query()`` method will send the ``AUXIn:PRObe:RANge?`` query.
            - Using the ``.verify(value)`` method will send the ``AUXIn:PRObe:RANge?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AUXIn:PRObe:RANge value`` command.

        SCPI Syntax:
            ```
            - AUXIn:PRObe:RANge {ATTEN1X|ATTEN1_25X|ATTEN1_5X|ATTEN1_75X|ATTEN2X|ATTEN2_5X|ATTEN3X|ATTEN3_5X|ATTEN4X|ATTEN4_5X|ATTEN5X|ATTEN5_5X|ATTEN6X|ATTEN6_5X|ATTEN7X|ATTEN7_5X|ATTEN8X|ATTEN8_5X|ATTEN9X|ATTEN9_5X|ATTEN10X|ATTEN12_5X|ATTEN15X|ATTEN17_5X|ATTEN20X|ATTEN25X|ATTEN30X|ATTEN35X|ATTEN40X|ATTEN45X|ATTEN50X|ATTEN55X|ATTEN60X|ATTEN65X|ATTEN70X|ATTEN75X|ATTEN80X|ATTEN85X|ATTEN90X|ATTEN95X|ATTEN100X|ATTEN125X|ATTEN150X|ATTEN175X|ATTEN200X|ATTEN250X|ATTEN300X|ATTEN350X|ATTEN400X|ATTEN450X|ATTEN500X|ATTEN550X|ATTEN600X|ATTEN650X|ATTEN700X|ATTEN750X|ATTEN800X|ATTEN850X|ATTEN900X|ATTEN950X|ATTEN1000X}
            - AUXIn:PRObe:RANge?
            ```
        """  # noqa: E501
        return self._range

    @property
    def resistance(self) -> AuxinProbeResistance:
        """Return the ``AUXIn:PRObe:RESistance`` command.

        Description:
            - This query-only command returns the resistance factor of the probe that is attached to
              the AUX In input.

        Usage:
            - Using the ``.query()`` method will send the ``AUXIn:PRObe:RESistance?`` query.
            - Using the ``.verify(value)`` method will send the ``AUXIn:PRObe:RESistance?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - AUXIn:PRObe:RESistance?
            ```
        """
        return self._resistance

    @property
    def set_(self) -> AuxinProbeSet:
        """Return the ``AUXIn:PRObe:SET`` command.

        Description:
            - This command sets or queries aspects of probe accessory user interfaces, for example
              probe attenuation factors. The available arguments for this command will vary
              depending on the accessory you attach to the instrument auxiliary trigger input. For
              the P7260 probe, you can select between two attenuation factors using either this GPIB
              command or the push switch on the probe. The probe enables the relevant path and
              adjusts the settings based on the characteristics of the path in use. The probe signal
              path selection is not kept in persistent storage. The probe will lose the selection if
              you reboot the instrument or remove the probe. Also, the instrument does not store the
              selection in the save/recall setup operation.

        Usage:
            - Using the ``.query()`` method will send the ``AUXIn:PRObe:SET?`` query.
            - Using the ``.verify(value)`` method will send the ``AUXIn:PRObe:SET?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AUXIn:PRObe:SET value`` command.

        SCPI Syntax:
            ```
            - AUXIn:PRObe:SET <QString>
            - AUXIn:PRObe:SET?
            ```

        Info:
            - ``<QString>`` is a quoted string representing a settable aspect of the attached
              accessory.
            - ``ATTENUATION 5X`` sets the P7260 probe to ±0.75 V dynamic range with 6 GHz bandwidth
              and 5X attenuation.
            - ``ATTENUATION 25X`` sets the P7260 probe to ±3 V dynamic range with 6 GHz bandwidth
              and 25X attenuation.
            - ``VTERMsource AUTO`` sets the P7380SMA probe voltage termination source to auto.
            - ``VTERMsource INTernal`` sets the P7380SMA probe voltage termination source to
              internal.
            - ``VTERMsource EXTernal`` sets the P7380SMA probe voltage termination source to
              external.
        """
        return self._set

    @property
    def signal(self) -> AuxinProbeSignal:
        """Return the ``AUXIn:PRObe:SIGnal`` command.

        Description:
            - This command sets or queries aspects of probe accessory user interfaces. The available
              arguments for this command will vary depending on the accessory you attach to the
              instrument.

        Usage:
            - Using the ``.query()`` method will send the ``AUXIn:PRObe:SIGnal?`` query.
            - Using the ``.verify(value)`` method will send the ``AUXIn:PRObe:SIGnal?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AUXIn:PRObe:SIGnal value`` command.

        SCPI Syntax:
            ```
            - AUXIn:PRObe:SIGnal {PASS|BYPass}
            - AUXIn:PRObe:SIGnal?
            ```

        Info:
            - ``PASS`` opens a relay passing your signal to the instrument.
            - ``BYPass`` closes a relay preventing your signal from reaching the instrument. During
              probe degauss, the signal should be bypassed.
        """
        return self._signal

    @property
    def tiptype(self) -> AuxinProbeTiptype:
        """Return the ``AUXIn:PRObe:TIPtype`` command.

        Description:
            - This command sets or queries the type of probe tip being used.

        Usage:
            - Using the ``.query()`` method will send the ``AUXIn:PRObe:TIPtype?`` query.
            - Using the ``.verify(value)`` method will send the ``AUXIn:PRObe:TIPtype?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AUXIn:PRObe:TIPtype value``
              command.

        SCPI Syntax:
            ```
            - AUXIn:PRObe:TIPtype {HBWStraightflex|OTHer}
            - AUXIn:PRObe:TIPtype?
            ```

        Info:
            - ``HBWStraightflex`` lets the instrument know you are using a high bandwidth
              straight-flex probe tip.
            - ``OTHer`` lets the instrument know you are not using a high bandwidth straight-flex
              probe tip.
        """
        return self._tiptype

    @property
    def units(self) -> AuxinProbeUnits:
        """Return the ``AUXIn:PRObe:UNIts`` command.

        Description:
            - This query-only command returns a string describing the units of measure for the probe
              attached to the AUX In input.

        Usage:
            - Using the ``.query()`` method will send the ``AUXIn:PRObe:UNIts?`` query.
            - Using the ``.verify(value)`` method will send the ``AUXIn:PRObe:UNIts?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - AUXIn:PRObe:UNIts?
            ```
        """
        return self._units


class AuxinProbefuncExtunits(SCPICmdWrite, SCPICmdRead):
    """The ``AUXIn:PROBEFunc:EXTUnits`` command.

    Description:
        - This command sets the unit of measurement for the external attenuator of the AUX In input.
          There is also a corresponding query that returns the user-specified unit of measurement
          for the external attenuator. Unless these units are set to the factory default string
          value of 'None', they become the attenuated units of measurement for the input. It is
          assumed that the probe connected to the input is of the correct type to receive the output
          of the user's external transducer or network.

    Usage:
        - Using the ``.query()`` method will send the ``AUXIn:PROBEFunc:EXTUnits?`` query.
        - Using the ``.verify(value)`` method will send the ``AUXIn:PROBEFunc:EXTUnits?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AUXIn:PROBEFunc:EXTUnits value``
          command.

    SCPI Syntax:
        ```
        - AUXIn:PROBEFunc:EXTUnits <QString>
        - AUXIn:PROBEFunc:EXTUnits?
        ```

    Info:
        - ``<QString>`` can contain a string of up to eight characters to indicate the attenuation
          unit of measurement for the AUX In input. However, most instrument attenuators only
          display the first two characters.
    """

    _WRAP_ARG_WITH_QUOTES = True


class AuxinProbefuncExtdbatten(SCPICmdWrite, SCPICmdRead):
    """The ``AUXIn:PROBEFunc:EXTDBatten`` command.

    Description:
        - This command sets the input-output ratio (expressed in decibel units) of external
          attenuation or gain between the signal and the instrument AUX In input. The query form of
          this command returns the user-specified attenuation in decibels: 1X = 0 dB, 10X = 20 dB,
          100X = 40 dB, and similar things. This command is equivalent to selecting Attenuation from
          the Vertical menu, and then either viewing or setting Ext Att(dB).

    Usage:
        - Using the ``.query()`` method will send the ``AUXIn:PROBEFunc:EXTDBatten?`` query.
        - Using the ``.verify(value)`` method will send the ``AUXIn:PROBEFunc:EXTDBatten?`` query
          and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AUXIn:PROBEFunc:EXTDBatten value``
          command.

    SCPI Syntax:
        ```
        - AUXIn:PROBEFunc:EXTDBatten <NR3>
        - AUXIn:PROBEFunc:EXTDBatten?
        ```

    Info:
        - ``<NR3>`` is the attenuation value, which is specified in the range from -200.00 dB to
          200.00 dB.
    """


class AuxinProbefuncExtatten(SCPICmdWrite, SCPICmdRead):
    """The ``AUXIn:PROBEFunc:EXTAtten`` command.

    Description:
        - This command sets the input-output ratio, of external attenuation or gain, between the
          signal and the AUX In input. The query form of this command returns the user-specified
          attenuation. Note that this command deals with an attenuation factor, not a gain factor,
          unlike ``CHX:PROBE`` (This command returns a value independent of the external
          attenuation). For example, if you specify a 20X attenuation factor, the commands return
          the following values (assuming that a 1x probe is presently attached, since the external
          attenuation is used in combination with the probe attenuation): ``AUXIN:PROBE:EXTA?``
          20.00E+0 ``AUXIN:PROBE?`` 1.0E+0 This command is equivalent to selecting Attenuation from
          the Vertical menu, and then either viewing or setting Ext Atten.

    Usage:
        - Using the ``.query()`` method will send the ``AUXIn:PROBEFunc:EXTAtten?`` query.
        - Using the ``.verify(value)`` method will send the ``AUXIn:PROBEFunc:EXTAtten?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AUXIn:PROBEFunc:EXTAtten value``
          command.

    SCPI Syntax:
        ```
        - AUXIn:PROBEFunc:EXTAtten <NR3>
        - AUXIn:PROBEFunc:EXTAtten?
        ```

    Info:
        - ``<NR3>`` is the attenuation value, which is specified as a multiplier in the range from
          1.00E-10 to 1.00E+10.
    """


class AuxinProbefunc(SCPICmdRead):
    """The ``AUXIn:PROBEFunc`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``AUXIn:PROBEFunc?`` query.
        - Using the ``.verify(value)`` method will send the ``AUXIn:PROBEFunc?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.extatten``: The ``AUXIn:PROBEFunc:EXTAtten`` command.
        - ``.extdbatten``: The ``AUXIn:PROBEFunc:EXTDBatten`` command.
        - ``.extunits``: The ``AUXIn:PROBEFunc:EXTUnits`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._extatten = AuxinProbefuncExtatten(device, f"{self._cmd_syntax}:EXTAtten")
        self._extdbatten = AuxinProbefuncExtdbatten(device, f"{self._cmd_syntax}:EXTDBatten")
        self._extunits = AuxinProbefuncExtunits(device, f"{self._cmd_syntax}:EXTUnits")

    @property
    def extatten(self) -> AuxinProbefuncExtatten:
        """Return the ``AUXIn:PROBEFunc:EXTAtten`` command.

        Description:
            - This command sets the input-output ratio, of external attenuation or gain, between the
              signal and the AUX In input. The query form of this command returns the user-specified
              attenuation. Note that this command deals with an attenuation factor, not a gain
              factor, unlike ``CHX:PROBE`` (This command returns a value independent of the external
              attenuation). For example, if you specify a 20X attenuation factor, the commands
              return the following values (assuming that a 1x probe is presently attached, since the
              external attenuation is used in combination with the probe attenuation):
              ``AUXIN:PROBE:EXTA?`` 20.00E+0 ``AUXIN:PROBE?`` 1.0E+0 This command is equivalent to
              selecting Attenuation from the Vertical menu, and then either viewing or setting Ext
              Atten.

        Usage:
            - Using the ``.query()`` method will send the ``AUXIn:PROBEFunc:EXTAtten?`` query.
            - Using the ``.verify(value)`` method will send the ``AUXIn:PROBEFunc:EXTAtten?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AUXIn:PROBEFunc:EXTAtten value``
              command.

        SCPI Syntax:
            ```
            - AUXIn:PROBEFunc:EXTAtten <NR3>
            - AUXIn:PROBEFunc:EXTAtten?
            ```

        Info:
            - ``<NR3>`` is the attenuation value, which is specified as a multiplier in the range
              from 1.00E-10 to 1.00E+10.
        """
        return self._extatten

    @property
    def extdbatten(self) -> AuxinProbefuncExtdbatten:
        """Return the ``AUXIn:PROBEFunc:EXTDBatten`` command.

        Description:
            - This command sets the input-output ratio (expressed in decibel units) of external
              attenuation or gain between the signal and the instrument AUX In input. The query form
              of this command returns the user-specified attenuation in decibels: 1X = 0 dB, 10X =
              20 dB, 100X = 40 dB, and similar things. This command is equivalent to selecting
              Attenuation from the Vertical menu, and then either viewing or setting Ext Att(dB).

        Usage:
            - Using the ``.query()`` method will send the ``AUXIn:PROBEFunc:EXTDBatten?`` query.
            - Using the ``.verify(value)`` method will send the ``AUXIn:PROBEFunc:EXTDBatten?``
              query and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AUXIn:PROBEFunc:EXTDBatten value``
              command.

        SCPI Syntax:
            ```
            - AUXIn:PROBEFunc:EXTDBatten <NR3>
            - AUXIn:PROBEFunc:EXTDBatten?
            ```

        Info:
            - ``<NR3>`` is the attenuation value, which is specified in the range from -200.00 dB to
              200.00 dB.
        """
        return self._extdbatten

    @property
    def extunits(self) -> AuxinProbefuncExtunits:
        """Return the ``AUXIn:PROBEFunc:EXTUnits`` command.

        Description:
            - This command sets the unit of measurement for the external attenuator of the AUX In
              input. There is also a corresponding query that returns the user-specified unit of
              measurement for the external attenuator. Unless these units are set to the factory
              default string value of 'None', they become the attenuated units of measurement for
              the input. It is assumed that the probe connected to the input is of the correct type
              to receive the output of the user's external transducer or network.

        Usage:
            - Using the ``.query()`` method will send the ``AUXIn:PROBEFunc:EXTUnits?`` query.
            - Using the ``.verify(value)`` method will send the ``AUXIn:PROBEFunc:EXTUnits?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AUXIn:PROBEFunc:EXTUnits value``
              command.

        SCPI Syntax:
            ```
            - AUXIn:PROBEFunc:EXTUnits <QString>
            - AUXIn:PROBEFunc:EXTUnits?
            ```

        Info:
            - ``<QString>`` can contain a string of up to eight characters to indicate the
              attenuation unit of measurement for the AUX In input. However, most instrument
              attenuators only display the first two characters.
        """
        return self._extunits


class AuxinOffset(SCPICmdWrite, SCPICmdRead):
    """The ``AUXIn:OFFSet`` command.

    Description:
        - This command sets or queries the vertical offset for the AUX In input. This command is
          equivalent to selecting Offset from the Vertical menu. This command offsets the vertical
          acquisition window (moves the level at the vertical center of the acquisition window) for
          the input. Visualize offset as scrolling the acquisition window towards the top of a large
          signal for increased offset values, and scrolling towards the bottom for decreased offset
          values. The resolution of the vertical window sets the offset increment for this control.
          Offset adjusts only the vertical center of the acquisition window to help determine what
          data is acquired. The instrument always displays the input signal minus the offset value.
          The channel reference marker will move to the vertical graticule position given by the
          negative of the offset value divided by the scale factor, unless that position is
          off-screen. If the computed coordinate for the reference mark is off-screen, the mark
          moves to the nearest screen limit and changes from a right-pointing arrow ( →) to an arrow
          pointing in the appropriate off-screen direction.

    Usage:
        - Using the ``.query()`` method will send the ``AUXIn:OFFSet?`` query.
        - Using the ``.verify(value)`` method will send the ``AUXIn:OFFSet?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AUXIn:OFFSet value`` command.

    SCPI Syntax:
        ```
        - AUXIn:OFFSet <NR3>
        - AUXIn:OFFSet?
        ```

    Info:
        - ``<NR3>`` is the offset value for the specified channel.
    """


class AuxinCoupling(SCPICmdWrite, SCPICmdRead):
    """The ``AUXIn:COUPling`` command.

    Description:
        - This command sets or queries the input attenuator coupling setting for the AUX In input.
          The coupling will change with an appropriate probe attached. This command is equivalent to
          selecting Coupling from the Vertical menu.

    Usage:
        - Using the ``.query()`` method will send the ``AUXIn:COUPling?`` query.
        - Using the ``.verify(value)`` method will send the ``AUXIn:COUPling?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AUXIn:COUPling value`` command.

    SCPI Syntax:
        ```
        - AUXIn:COUPling <NR3>
        - AUXIn:COUPling?
        ```

    Info:
        - ``<NR3>`` is one of the supported input attenuator couplings on the attached probe.
    """


class AuxinBandwidth(SCPICmdWrite, SCPICmdRead):
    """The ``AUXIn:BANdwidth`` command.

    Description:
        - This command sets or queries the selectable low-pass bandwidth limit filter of the AUX In
          input. The bandwidth will change with an appropriate probe attached. This is equivalent to
          selecting Bandwidth from the Vertical menu. The query form of this command always returns
          the maximum bandwidth of the AUX In input.

    Usage:
        - Using the ``.query()`` method will send the ``AUXIn:BANdwidth?`` query.
        - Using the ``.verify(value)`` method will send the ``AUXIn:BANdwidth?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AUXIn:BANdwidth value`` command.

    SCPI Syntax:
        ```
        - AUXIn:BANdwidth <NR3>
        - AUXIn:BANdwidth?
        ```

    Info:
        - ``<NR3>`` is one of the supported bandwidths on the attached probe.
    """


class Auxin(SCPICmdRead):
    """The ``AUXIn`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``AUXIn?`` query.
        - Using the ``.verify(value)`` method will send the ``AUXIn?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.bandwidth``: The ``AUXIn:BANdwidth`` command.
        - ``.coupling``: The ``AUXIn:COUPling`` command.
        - ``.offset``: The ``AUXIn:OFFSet`` command.
        - ``.probefunc``: The ``AUXIn:PROBEFunc`` command tree.
        - ``.probe``: The ``AUXIn:PRObe`` command tree.
        - ``.vterm``: The ``AUXIn:VTERm`` command tree.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "AUXIn") -> None:
        super().__init__(device, cmd_syntax)
        self._bandwidth = AuxinBandwidth(device, f"{self._cmd_syntax}:BANdwidth")
        self._coupling = AuxinCoupling(device, f"{self._cmd_syntax}:COUPling")
        self._offset = AuxinOffset(device, f"{self._cmd_syntax}:OFFSet")
        self._probefunc = AuxinProbefunc(device, f"{self._cmd_syntax}:PROBEFunc")
        self._probe = AuxinProbe(device, f"{self._cmd_syntax}:PRObe")
        self._vterm = AuxinVterm(device, f"{self._cmd_syntax}:VTERm")

    @property
    def bandwidth(self) -> AuxinBandwidth:
        """Return the ``AUXIn:BANdwidth`` command.

        Description:
            - This command sets or queries the selectable low-pass bandwidth limit filter of the AUX
              In input. The bandwidth will change with an appropriate probe attached. This is
              equivalent to selecting Bandwidth from the Vertical menu. The query form of this
              command always returns the maximum bandwidth of the AUX In input.

        Usage:
            - Using the ``.query()`` method will send the ``AUXIn:BANdwidth?`` query.
            - Using the ``.verify(value)`` method will send the ``AUXIn:BANdwidth?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AUXIn:BANdwidth value`` command.

        SCPI Syntax:
            ```
            - AUXIn:BANdwidth <NR3>
            - AUXIn:BANdwidth?
            ```

        Info:
            - ``<NR3>`` is one of the supported bandwidths on the attached probe.
        """
        return self._bandwidth

    @property
    def coupling(self) -> AuxinCoupling:
        """Return the ``AUXIn:COUPling`` command.

        Description:
            - This command sets or queries the input attenuator coupling setting for the AUX In
              input. The coupling will change with an appropriate probe attached. This command is
              equivalent to selecting Coupling from the Vertical menu.

        Usage:
            - Using the ``.query()`` method will send the ``AUXIn:COUPling?`` query.
            - Using the ``.verify(value)`` method will send the ``AUXIn:COUPling?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AUXIn:COUPling value`` command.

        SCPI Syntax:
            ```
            - AUXIn:COUPling <NR3>
            - AUXIn:COUPling?
            ```

        Info:
            - ``<NR3>`` is one of the supported input attenuator couplings on the attached probe.
        """
        return self._coupling

    @property
    def offset(self) -> AuxinOffset:
        """Return the ``AUXIn:OFFSet`` command.

        Description:
            - This command sets or queries the vertical offset for the AUX In input. This command is
              equivalent to selecting Offset from the Vertical menu. This command offsets the
              vertical acquisition window (moves the level at the vertical center of the acquisition
              window) for the input. Visualize offset as scrolling the acquisition window towards
              the top of a large signal for increased offset values, and scrolling towards the
              bottom for decreased offset values. The resolution of the vertical window sets the
              offset increment for this control. Offset adjusts only the vertical center of the
              acquisition window to help determine what data is acquired. The instrument always
              displays the input signal minus the offset value. The channel reference marker will
              move to the vertical graticule position given by the negative of the offset value
              divided by the scale factor, unless that position is off-screen. If the computed
              coordinate for the reference mark is off-screen, the mark moves to the nearest screen
              limit and changes from a right-pointing arrow ( →) to an arrow pointing in the
              appropriate off-screen direction.

        Usage:
            - Using the ``.query()`` method will send the ``AUXIn:OFFSet?`` query.
            - Using the ``.verify(value)`` method will send the ``AUXIn:OFFSet?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``AUXIn:OFFSet value`` command.

        SCPI Syntax:
            ```
            - AUXIn:OFFSet <NR3>
            - AUXIn:OFFSet?
            ```

        Info:
            - ``<NR3>`` is the offset value for the specified channel.
        """
        return self._offset

    @property
    def probefunc(self) -> AuxinProbefunc:
        """Return the ``AUXIn:PROBEFunc`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``AUXIn:PROBEFunc?`` query.
            - Using the ``.verify(value)`` method will send the ``AUXIn:PROBEFunc?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.extatten``: The ``AUXIn:PROBEFunc:EXTAtten`` command.
            - ``.extdbatten``: The ``AUXIn:PROBEFunc:EXTDBatten`` command.
            - ``.extunits``: The ``AUXIn:PROBEFunc:EXTUnits`` command.
        """
        return self._probefunc

    @property
    def probe(self) -> AuxinProbe:
        """Return the ``AUXIn:PRObe`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``AUXIn:PRObe?`` query.
            - Using the ``.verify(value)`` method will send the ``AUXIn:PRObe?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.autozero``: The ``AUXIn:PRObe:AUTOZero`` command.
            - ``.degauss``: The ``AUXIn:PRObe:DEGAUSS`` command.
            - ``.forcedrange``: The ``AUXIn:PRObe:FORCEDRange`` command.
            - ``.gain``: The ``AUXIn:PRObe:GAIN`` command.
            - ``.id``: The ``AUXIn:PRObe:ID`` command tree.
            - ``.inputmode``: The ``AUXIn:PRObe:INPUTMode`` command.
            - ``.range``: The ``AUXIn:PRObe:RANge`` command.
            - ``.resistance``: The ``AUXIn:PRObe:RESistance`` command.
            - ``.set``: The ``AUXIn:PRObe:SET`` command.
            - ``.signal``: The ``AUXIn:PRObe:SIGnal`` command.
            - ``.tiptype``: The ``AUXIn:PRObe:TIPtype`` command.
            - ``.units``: The ``AUXIn:PRObe:UNIts`` command.
        """
        return self._probe

    @property
    def vterm(self) -> AuxinVterm:
        """Return the ``AUXIn:VTERm`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``AUXIn:VTERm?`` query.
            - Using the ``.verify(value)`` method will send the ``AUXIn:VTERm?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.dual``: The ``AUXIn:VTERm:DUAL`` command tree.
        """
        return self._vterm



"""The histogram commands module.

These commands are used in the following models:
MDO3

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - HIStogram:BOX <NR3>, <NR3>, <NR3>, <NR3>
    - HIStogram:BOX?
    - HIStogram:BOXPcnt <NR2>, <NR2>, <NR2>, <NR2>
    - HIStogram:BOXPcnt?
    - HIStogram:COUNt RESET
    - HIStogram:DATa?
    - HIStogram:DISplay {OFF|LOG|LINEAr}
    - HIStogram:DISplay?
    - HIStogram:END?
    - HIStogram:MODe {HORizontal|VERTical|OFF}
    - HIStogram:MODe?
    - HIStogram:SOUrce {CH<x>|MATH|REF<x>}
    - HIStogram:SOUrce?
    - HIStogram:STARt?
    - HIStogram?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class HistogramStart(SCPICmdRead):
    """The ``HIStogram:STARt`` command.

    Description:
        - Returns the time (horizontal) or vertical units value (vertical) of the first bin where
          the histogram data starts. The ``HISTOGRAM:MODE`` must be either HORizontal or VERTical
          for a value to be returned. If the mode is OFF, an error event is set and nothing is
          returned. If the ``HISTOGRAM:MODE`` is HORizontal, the value returned is the time of the
          left bin. If the ``HISTOGRAM:MODE`` is VERTical the value returned is the vertical units
          value of the top bin. The returned value is an <NR3>.

    Usage:
        - Using the ``.query()`` method will send the ``HIStogram:STARt?`` query.
        - Using the ``.verify(value)`` method will send the ``HIStogram:STARt?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - HIStogram:STARt?
        ```
    """


class HistogramSource(SCPICmdWrite, SCPICmdRead):
    """The ``HIStogram:SOUrce`` command.

    Description:
        - This command sets or queries which source will be compared against the histogram box when
          the histogram testing is enabled.

    Usage:
        - Using the ``.query()`` method will send the ``HIStogram:SOUrce?`` query.
        - Using the ``.verify(value)`` method will send the ``HIStogram:SOUrce?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``HIStogram:SOUrce value`` command.

    SCPI Syntax:
        ```
        - HIStogram:SOUrce {CH<x>|MATH|REF<x>}
        - HIStogram:SOUrce?
        ```

    Info:
        - ``CH<x>`` selects the analog channel waveform to use as the source for the histogram. The
          x variable can be expressed as an integer ranging from 1 through 4.
        - ``MATH`` selects the math waveform as the source for the histogram.
        - ``REF<x>`` selects a reference waveform as the source for the histogram. The x variable
          can be expressed as an integer ranging from 1 through 4.
    """


class HistogramMode(SCPICmdWrite, SCPICmdRead):
    """The ``HIStogram:MODe`` command.

    Description:
        - This command selects the type of histogram to create or disables the histogram display.
          The query form either returns the current histogram type or that the histogram display is
          disabled.

    Usage:
        - Using the ``.query()`` method will send the ``HIStogram:MODe?`` query.
        - Using the ``.verify(value)`` method will send the ``HIStogram:MODe?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``HIStogram:MODe value`` command.

    SCPI Syntax:
        ```
        - HIStogram:MODe {HORizontal|VERTical|OFF}
        - HIStogram:MODe?
        ```

    Info:
        - ``HORizontal`` enables a horizontally positioned histogram that shows time distribution.
        - ``VERTical`` enables a vertically positioned histogram that shows a voltage distribution,
          or another distribution such as amperes.
        - ``OFF`` disables the collection of the histogram measurement.
    """


class HistogramEnd(SCPICmdRead):
    """The ``HIStogram:END`` command.

    Description:
        - Returns the time (horizontal) or vertical units value (vertical) of the last bin where the
          histogram data ends. The ``HISTOGRAM:MODE`` must be either HORizontal or VERTical for a
          value to be returned. If the mode is OFF, an error event is set and nothing is returned.
          If the ``HISTOGRAM:MODE`` is HORizontal, the value returned is the time of the right bin.
          If the ``HISTOGRAM:MODE`` is VERTical the value returned is the vertical units value of
          the bottom bin. The returned value is an <NR3>.

    Usage:
        - Using the ``.query()`` method will send the ``HIStogram:END?`` query.
        - Using the ``.verify(value)`` method will send the ``HIStogram:END?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - HIStogram:END?
        ```
    """


class HistogramDisplay(SCPICmdWrite, SCPICmdRead):
    """The ``HIStogram:DISplay`` command.

    Description:
        - This command sets the scaling of the histogram data display to be the count of each
          histogram bin or the log of that count. The default scaling is linear.

    Usage:
        - Using the ``.query()`` method will send the ``HIStogram:DISplay?`` query.
        - Using the ``.verify(value)`` method will send the ``HIStogram:DISplay?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``HIStogram:DISplay value`` command.

    SCPI Syntax:
        ```
        - HIStogram:DISplay {OFF|LOG|LINEAr}
        - HIStogram:DISplay?
        ```

    Info:
        - ``<LOG>`` sets the histogram display to logarithmic scaling.
        - ``<LINEAr>`` sets the histogram display to linear scaling. This is the default setting.
    """


class HistogramData(SCPICmdRead):
    """The ``HIStogram:DATa`` command.

    Description:
        - This query returns the histogram data when ``HIStogram:MODe`` is HORizontal or VERTical.
          If the mode is OFF, then no data is returned and an error event is set. The data values
          returned for this query represent the number of times the histogram source waveform
          samples were coincident with a particular histogram bin. For vertical histograms, this
          query returns 256 values, representing the number of times the histogram source waveform
          samples were coincident with each of the 256 digitizing levels. Of these 256 values, the
          first 2 and last 3 are always 0, as they represent digitizing levels that fall above and
          beneath the waveform graticule, respectively. For horizontal histograms, this query
          returns 1000 values, representing the number of times the histogram source waveform
          samples were coincident with each horizontal pixel column. The time of occurrence for each
          of the horizontal bins can be derived using the ``HISTOGRAM:START`` and ``HISTOGRAM:END``
          queries.

    Usage:
        - Using the ``.query()`` method will send the ``HIStogram:DATa?`` query.
        - Using the ``.verify(value)`` method will send the ``HIStogram:DATa?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - HIStogram:DATa?
        ```
    """


class HistogramCount(SCPICmdWrite):
    """The ``HIStogram:COUNt`` command.

    Description:
        - This command (no query form) clears the count and statistics for the histogram and the
          histogram source data. If the histogram is on, then counting restarts.

    Usage:
        - Using the ``.write(value)`` method will send the ``HIStogram:COUNt value`` command.

    SCPI Syntax:
        ```
        - HIStogram:COUNt RESET
        ```

    Info:
        - ``RESET``
    """


class HistogramBoxpcnt(SCPICmdWrite, SCPICmdRead):
    """The ``HIStogram:BOXPcnt`` command.

    Description:
        - This command specifies the histogram box coordinates in terms of percentages of the full
          screen extents of the source waveform. The arguments refer to the left (percent of the
          horizontal screen extent), top (percent of the vertical screen extent), right (percent of
          the horizontal screen extent), bottom (percent of the vertical screen extent). The valid
          range for these values is 0.0 to 100.0.

    Usage:
        - Using the ``.query()`` method will send the ``HIStogram:BOXPcnt?`` query.
        - Using the ``.verify(value)`` method will send the ``HIStogram:BOXPcnt?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``HIStogram:BOXPcnt value`` command.

    SCPI Syntax:
        ```
        - HIStogram:BOXPcnt <NR2>, <NR2>, <NR2>, <NR2>
        - HIStogram:BOXPcnt?
        ```

    Info:
        - ``<NR3>`` is a floating point number that specifies the left position of the histogram box
          in percentage coordinates. The default value is 20%.
        - ``<NR3>`` specifies the top position of the histogram box in percentage coordinates. The
          default value is 80%.
        - ``<NR3>`` specifies the right position of the histogram box in percentage coordinates. The
          default value is 80%.
        - ``<NR3>`` specifies the bottom position of the histogram box in percentage coordinates.
          The default value is 20%.
    """


class HistogramBox(SCPICmdWrite, SCPICmdRead):
    """The ``HIStogram:BOX`` command.

    Description:
        - This command specifies the histogram box coordinates in terms of source waveform vertical
          and horizontal units.

    Usage:
        - Using the ``.query()`` method will send the ``HIStogram:BOX?`` query.
        - Using the ``.verify(value)`` method will send the ``HIStogram:BOX?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``HIStogram:BOX value`` command.

    SCPI Syntax:
        ```
        - HIStogram:BOX <NR3>, <NR3>, <NR3>, <NR3>
        - HIStogram:BOX?
        ```

    Info:
        - ``<NR3>`` is a floating point number that specifies the left position of the histogram box
          in source waveform horizontal units.
        - ``<NR3>`` specifies the top position of the histogram box in source waveform vertical
          units.
        - ``<NR3>`` specifies the right position of the histogram box in source waveform horizontal
          units.
        - ``<NR3>`` specifies the bottom position of the histogram box in source waveform vertical
          units.
    """


#  pylint: disable=too-many-instance-attributes
class Histogram(SCPICmdRead):
    """The ``HIStogram`` command.

    Description:
        - This query-only query returns all histogram parameters; it queries the state of all
          histogram parameters that the user can set. This command is equivalent to selecting
          Waveform Histograms from the Measure menu.

    Usage:
        - Using the ``.query()`` method will send the ``HIStogram?`` query.
        - Using the ``.verify(value)`` method will send the ``HIStogram?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - HIStogram?
        ```

    Properties:
        - ``.box``: The ``HIStogram:BOX`` command.
        - ``.boxpcnt``: The ``HIStogram:BOXPcnt`` command.
        - ``.count``: The ``HIStogram:COUNt`` command.
        - ``.data``: The ``HIStogram:DATa`` command.
        - ``.display``: The ``HIStogram:DISplay`` command.
        - ``.end``: The ``HIStogram:END`` command.
        - ``.mode``: The ``HIStogram:MODe`` command.
        - ``.source``: The ``HIStogram:SOUrce`` command.
        - ``.start``: The ``HIStogram:STARt`` command.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "HIStogram") -> None:
        super().__init__(device, cmd_syntax)
        self._box = HistogramBox(device, f"{self._cmd_syntax}:BOX")
        self._boxpcnt = HistogramBoxpcnt(device, f"{self._cmd_syntax}:BOXPcnt")
        self._count = HistogramCount(device, f"{self._cmd_syntax}:COUNt")
        self._data = HistogramData(device, f"{self._cmd_syntax}:DATa")
        self._display = HistogramDisplay(device, f"{self._cmd_syntax}:DISplay")
        self._end = HistogramEnd(device, f"{self._cmd_syntax}:END")
        self._mode = HistogramMode(device, f"{self._cmd_syntax}:MODe")
        self._source = HistogramSource(device, f"{self._cmd_syntax}:SOUrce")
        self._start = HistogramStart(device, f"{self._cmd_syntax}:STARt")

    @property
    def box(self) -> HistogramBox:
        """Return the ``HIStogram:BOX`` command.

        Description:
            - This command specifies the histogram box coordinates in terms of source waveform
              vertical and horizontal units.

        Usage:
            - Using the ``.query()`` method will send the ``HIStogram:BOX?`` query.
            - Using the ``.verify(value)`` method will send the ``HIStogram:BOX?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``HIStogram:BOX value`` command.

        SCPI Syntax:
            ```
            - HIStogram:BOX <NR3>, <NR3>, <NR3>, <NR3>
            - HIStogram:BOX?
            ```

        Info:
            - ``<NR3>`` is a floating point number that specifies the left position of the histogram
              box in source waveform horizontal units.
            - ``<NR3>`` specifies the top position of the histogram box in source waveform vertical
              units.
            - ``<NR3>`` specifies the right position of the histogram box in source waveform
              horizontal units.
            - ``<NR3>`` specifies the bottom position of the histogram box in source waveform
              vertical units.
        """
        return self._box

    @property
    def boxpcnt(self) -> HistogramBoxpcnt:
        """Return the ``HIStogram:BOXPcnt`` command.

        Description:
            - This command specifies the histogram box coordinates in terms of percentages of the
              full screen extents of the source waveform. The arguments refer to the left (percent
              of the horizontal screen extent), top (percent of the vertical screen extent), right
              (percent of the horizontal screen extent), bottom (percent of the vertical screen
              extent). The valid range for these values is 0.0 to 100.0.

        Usage:
            - Using the ``.query()`` method will send the ``HIStogram:BOXPcnt?`` query.
            - Using the ``.verify(value)`` method will send the ``HIStogram:BOXPcnt?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``HIStogram:BOXPcnt value`` command.

        SCPI Syntax:
            ```
            - HIStogram:BOXPcnt <NR2>, <NR2>, <NR2>, <NR2>
            - HIStogram:BOXPcnt?
            ```

        Info:
            - ``<NR3>`` is a floating point number that specifies the left position of the histogram
              box in percentage coordinates. The default value is 20%.
            - ``<NR3>`` specifies the top position of the histogram box in percentage coordinates.
              The default value is 80%.
            - ``<NR3>`` specifies the right position of the histogram box in percentage coordinates.
              The default value is 80%.
            - ``<NR3>`` specifies the bottom position of the histogram box in percentage
              coordinates. The default value is 20%.
        """
        return self._boxpcnt

    @property
    def count(self) -> HistogramCount:
        """Return the ``HIStogram:COUNt`` command.

        Description:
            - This command (no query form) clears the count and statistics for the histogram and the
              histogram source data. If the histogram is on, then counting restarts.

        Usage:
            - Using the ``.write(value)`` method will send the ``HIStogram:COUNt value`` command.

        SCPI Syntax:
            ```
            - HIStogram:COUNt RESET
            ```

        Info:
            - ``RESET``
        """
        return self._count

    @property
    def data(self) -> HistogramData:
        """Return the ``HIStogram:DATa`` command.

        Description:
            - This query returns the histogram data when ``HIStogram:MODe`` is HORizontal or
              VERTical. If the mode is OFF, then no data is returned and an error event is set. The
              data values returned for this query represent the number of times the histogram source
              waveform samples were coincident with a particular histogram bin. For vertical
              histograms, this query returns 256 values, representing the number of times the
              histogram source waveform samples were coincident with each of the 256 digitizing
              levels. Of these 256 values, the first 2 and last 3 are always 0, as they represent
              digitizing levels that fall above and beneath the waveform graticule, respectively.
              For horizontal histograms, this query returns 1000 values, representing the number of
              times the histogram source waveform samples were coincident with each horizontal pixel
              column. The time of occurrence for each of the horizontal bins can be derived using
              the ``HISTOGRAM:START`` and ``HISTOGRAM:END`` queries.

        Usage:
            - Using the ``.query()`` method will send the ``HIStogram:DATa?`` query.
            - Using the ``.verify(value)`` method will send the ``HIStogram:DATa?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - HIStogram:DATa?
            ```
        """
        return self._data

    @property
    def display(self) -> HistogramDisplay:
        """Return the ``HIStogram:DISplay`` command.

        Description:
            - This command sets the scaling of the histogram data display to be the count of each
              histogram bin or the log of that count. The default scaling is linear.

        Usage:
            - Using the ``.query()`` method will send the ``HIStogram:DISplay?`` query.
            - Using the ``.verify(value)`` method will send the ``HIStogram:DISplay?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``HIStogram:DISplay value`` command.

        SCPI Syntax:
            ```
            - HIStogram:DISplay {OFF|LOG|LINEAr}
            - HIStogram:DISplay?
            ```

        Info:
            - ``<LOG>`` sets the histogram display to logarithmic scaling.
            - ``<LINEAr>`` sets the histogram display to linear scaling. This is the default
              setting.
        """
        return self._display

    @property
    def end(self) -> HistogramEnd:
        """Return the ``HIStogram:END`` command.

        Description:
            - Returns the time (horizontal) or vertical units value (vertical) of the last bin where
              the histogram data ends. The ``HISTOGRAM:MODE`` must be either HORizontal or VERTical
              for a value to be returned. If the mode is OFF, an error event is set and nothing is
              returned. If the ``HISTOGRAM:MODE`` is HORizontal, the value returned is the time of
              the right bin. If the ``HISTOGRAM:MODE`` is VERTical the value returned is the
              vertical units value of the bottom bin. The returned value is an <NR3>.

        Usage:
            - Using the ``.query()`` method will send the ``HIStogram:END?`` query.
            - Using the ``.verify(value)`` method will send the ``HIStogram:END?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - HIStogram:END?
            ```
        """
        return self._end

    @property
    def mode(self) -> HistogramMode:
        """Return the ``HIStogram:MODe`` command.

        Description:
            - This command selects the type of histogram to create or disables the histogram
              display. The query form either returns the current histogram type or that the
              histogram display is disabled.

        Usage:
            - Using the ``.query()`` method will send the ``HIStogram:MODe?`` query.
            - Using the ``.verify(value)`` method will send the ``HIStogram:MODe?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``HIStogram:MODe value`` command.

        SCPI Syntax:
            ```
            - HIStogram:MODe {HORizontal|VERTical|OFF}
            - HIStogram:MODe?
            ```

        Info:
            - ``HORizontal`` enables a horizontally positioned histogram that shows time
              distribution.
            - ``VERTical`` enables a vertically positioned histogram that shows a voltage
              distribution, or another distribution such as amperes.
            - ``OFF`` disables the collection of the histogram measurement.
        """
        return self._mode

    @property
    def source(self) -> HistogramSource:
        """Return the ``HIStogram:SOUrce`` command.

        Description:
            - This command sets or queries which source will be compared against the histogram box
              when the histogram testing is enabled.

        Usage:
            - Using the ``.query()`` method will send the ``HIStogram:SOUrce?`` query.
            - Using the ``.verify(value)`` method will send the ``HIStogram:SOUrce?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``HIStogram:SOUrce value`` command.

        SCPI Syntax:
            ```
            - HIStogram:SOUrce {CH<x>|MATH|REF<x>}
            - HIStogram:SOUrce?
            ```

        Info:
            - ``CH<x>`` selects the analog channel waveform to use as the source for the histogram.
              The x variable can be expressed as an integer ranging from 1 through 4.
            - ``MATH`` selects the math waveform as the source for the histogram.
            - ``REF<x>`` selects a reference waveform as the source for the histogram. The x
              variable can be expressed as an integer ranging from 1 through 4.
        """
        return self._source

    @property
    def start(self) -> HistogramStart:
        """Return the ``HIStogram:STARt`` command.

        Description:
            - Returns the time (horizontal) or vertical units value (vertical) of the first bin
              where the histogram data starts. The ``HISTOGRAM:MODE`` must be either HORizontal or
              VERTical for a value to be returned. If the mode is OFF, an error event is set and
              nothing is returned. If the ``HISTOGRAM:MODE`` is HORizontal, the value returned is
              the time of the left bin. If the ``HISTOGRAM:MODE`` is VERTical the value returned is
              the vertical units value of the top bin. The returned value is an <NR3>.

        Usage:
            - Using the ``.query()`` method will send the ``HIStogram:STARt?`` query.
            - Using the ``.verify(value)`` method will send the ``HIStogram:STARt?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - HIStogram:STARt?
            ```
        """
        return self._start



"""The afgcontrol commands module.

These commands are used in the following models:
AFG3K, AFG3KB, AFG3KC

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - AFGControl:CSCopy {CH1|CH2},{CH1|CH2}
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class AfgcontrolCscopy(SCPICmdWrite):
    """The ``AFGControl:CSCopy`` command.

    Description:
        - This command copies setup parameters for one channel to another channel. If your arbitrary
          function generator is a single-channel model, this command is not supported.

    Usage:
        - Using the ``.write(value)`` method will send the ``AFGControl:CSCopy value`` command.

    SCPI Syntax:
        ```
        - AFGControl:CSCopy {CH1|CH2},{CH1|CH2}
        ```
    """


class Afgcontrol(SCPICmdRead):
    """The ``AFGControl`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``AFGControl?`` query.
        - Using the ``.verify(value)`` method will send the ``AFGControl?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.cscopy``: The ``AFGControl:CSCopy`` command.
    """

    def __init__(
        self, device: Optional["PIControl"] = None, cmd_syntax: str = "AFGControl"
    ) -> None:
        super().__init__(device, cmd_syntax)
        self._cscopy = AfgcontrolCscopy(device, f"{self._cmd_syntax}:CSCopy")

    @property
    def cscopy(self) -> AfgcontrolCscopy:
        """Return the ``AFGControl:CSCopy`` command.

        Description:
            - This command copies setup parameters for one channel to another channel. If your
              arbitrary function generator is a single-channel model, this command is not supported.

        Usage:
            - Using the ``.write(value)`` method will send the ``AFGControl:CSCopy value`` command.

        SCPI Syntax:
            ```
            - AFGControl:CSCopy {CH1|CH2},{CH1|CH2}
            ```
        """
        return self._cscopy



"""The miscellaneous commands module.

These commands are used in the following models:
DPO7AX, LPD6, MSO4, MSO4B, MSO5, MSO5B, MSO5LP, MSO6, MSO6B

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - *DDT {<Block>|<QString>}
    - *DDT?
    - *LRN?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class Lrn(SCPICmdRead):
    """The ``*LRN`` command.

    Description:
        - This query-only command returns the commands that list the instrument settings, allowing
          you to record or 'learn' the current instrument settings. You can use these commands to
          return the instrument to the state it was in when you made the ``*LRN?`` query. This
          command is identical to the SET? command.

    Usage:
        - Using the ``.query()`` method will send the ``*LRN?`` query.
        - Using the ``.verify(value)`` method will send the ``*LRN?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - *LRN?
        ```

    Info:
        - ``<QString>`` is the license nomenclature.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "*LRN") -> None:
        super().__init__(device, cmd_syntax)


class Ddt(SCPICmdWrite, SCPICmdRead):
    """The ``*DDT`` command.

    Description:
        - This command allows you to specify a command or a list of commands that are executed when
          the instrument receives a ``*TRG`` command. Define Device Trigger (``*DDT``) is a special
          alias that the ``*TRG`` command uses.

    Usage:
        - Using the ``.query()`` method will send the ``*DDT?`` query.
        - Using the ``.verify(value)`` method will send the ``*DDT?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``*DDT value`` command.

    SCPI Syntax:
        ```
        - *DDT {<Block>|<QString>}
        - *DDT?
        ```

    Info:
        - ``<Block>`` is a complete sequence of program messages. The messages can contain only
          valid commands that must be separated by semicolons and must follow all rules for
          concatenating commands. The sequence must be less than or equal to 80characters. The
          format of this argument is always returned as a query.
        - ``<QString>`` is a complete sequence of program messages. The messages can contain only
          valid commands that must be separated by semicolons and must follow all rules for
          concatenating commands. The sequence must be less than or equal to 80 characters.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "*DDT") -> None:
        super().__init__(device, cmd_syntax)



# pyright: reportConstantRedefinition=none
"""The slot commands module.

These commands are used in the following models:
DAQ6510

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Attributes and Functions:
    ```
    - slot[slot].amps.endchannel
    - slot[slot].amps.startchannel
    - slot[slot].analogoutput.endchannel
    - slot[slot].analogoutput.startchannel
    - slot[slot].commonsideohms
    - slot[slot].digitalio.endchannel
    - slot[slot].digitalio.startchannel
    - slot[slot].idn
    - slot[slot].isolated.endchannel
    - slot[slot].isolated.startchannel
    - slot[slot].matrix.columns
    - slot[slot].matrix.rows
    - slot[slot].maxvoltage
    - slot[slot].pseudocard
    - slot[slot].tempsensor
    - slot[slot].totalizer.endchannel
    - slot[slot].totalizer.startchannel
    - slot[slot].voltage.endchannel
    - slot[slot].voltage.startchannel
    ```
"""

from typing import Optional, TYPE_CHECKING, Union

from ..helpers import BaseTSPCmd, NoDeviceProvidedError, ValidatedDynamicNumberCmd

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.tsp_control import TSPControl


class SlotItemVoltage(BaseTSPCmd):
    """The ``slot[slot].voltage`` command tree.

    Info:
        - ``slot``, the slot number.

    Properties and methods:
        - ``.endchannel``: The ``slot[slot].voltage.endchannel`` attribute.
        - ``.startchannel``: The ``slot[slot].voltage.startchannel`` attribute.
    """

    @property
    def endchannel(self) -> str:
        """Access the ``slot[slot].voltage.endchannel`` attribute.

        Description:
            - This attribute indicates the last channel in the specified slot that supports voltage
              or 2-wire measurements.

        Usage:
            - Accessing this property will send the ``print(slot[slot].voltage.endchannel)`` query.

        TSP Syntax:
            ```
            - print(slot[slot].voltage.endchannel)
            ```

        Info:
            - ``slot``, the slot number.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".endchannel"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.endchannel)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.endchannel`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def startchannel(self) -> str:
        """Access the ``slot[slot].voltage.startchannel`` attribute.

        Description:
            - This attribute indicates the first channel in the specified slot that supports voltage
              or 2-wire measurements.

        Usage:
            - Accessing this property will send the ``print(slot[slot].voltage.startchannel)``
              query.

        TSP Syntax:
            ```
            - print(slot[slot].voltage.startchannel)
            ```

        Info:
            - ``slot``, the slot number.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".startchannel"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.startchannel)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.startchannel`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error


class SlotItemTotalizer(BaseTSPCmd):
    """The ``slot[slot].totalizer`` command tree.

    Info:
        - ``slot``, the slot number.

    Properties and methods:
        - ``.endchannel``: The ``slot[slot].totalizer.endchannel`` attribute.
        - ``.startchannel``: The ``slot[slot].totalizer.startchannel`` attribute.
    """

    @property
    def endchannel(self) -> str:
        """Access the ``slot[slot].totalizer.endchannel`` attribute.

        Description:
            - This attribute indicates the last totalizer channel in the specified slot.

        Usage:
            - Accessing this property will send the ``print(slot[slot].totalizer.endchannel)``
              query.

        TSP Syntax:
            ```
            - print(slot[slot].totalizer.endchannel)
            ```

        Info:
            - ``slot``, the slot number.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".endchannel"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.endchannel)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.endchannel`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def startchannel(self) -> str:
        """Access the ``slot[slot].totalizer.startchannel`` attribute.

        Description:
            - This attribute indicates the first totalizer channel in the specified slot.

        Usage:
            - Accessing this property will send the ``print(slot[slot].totalizer.startchannel)``
              query.

        TSP Syntax:
            ```
            - print(slot[slot].totalizer.startchannel)
            ```

        Info:
            - ``slot``, the slot number.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".startchannel"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.startchannel)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.startchannel`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error


class SlotItemMatrix(BaseTSPCmd):
    """The ``slot[slot].matrix`` command tree.

    Info:
        - ``slot``, the slot number.

    Properties and methods:
        - ``.columns``: The ``slot[slot].matrix.columns`` attribute.
        - ``.rows``: The ``slot[slot].matrix.rows`` attribute.
    """

    @property
    def columns(self) -> str:
        """Access the ``slot[slot].matrix.columns`` attribute.

        Description:
            - This attribute returns the number of columns in the matrix for the card in the
              specified slot.

        Usage:
            - Accessing this property will send the ``print(slot[slot].matrix.columns)`` query.

        TSP Syntax:
            ```
            - print(slot[slot].matrix.columns)
            ```

        Info:
            - ``slot``, the slot number.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".columns"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.columns)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.columns`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def rows(self) -> str:
        """Access the ``slot[slot].matrix.rows`` attribute.

        Description:
            - This attribute returns the number of rows in the matrix on the card in the specified
              slot.

        Usage:
            - Accessing this property will send the ``print(slot[slot].matrix.rows)`` query.

        TSP Syntax:
            ```
            - print(slot[slot].matrix.rows)
            ```

        Info:
            - ``slot``, the slot number (1 or 2).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".rows"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.rows)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.rows`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error


class SlotItemIsolated(BaseTSPCmd):
    """The ``slot[slot].isolated`` command tree.

    Info:
        - ``slot``, the slot number.

    Properties and methods:
        - ``.endchannel``: The ``slot[slot].isolated.endchannel`` attribute.
        - ``.startchannel``: The ``slot[slot].isolated.startchannel`` attribute.
    """

    @property
    def endchannel(self) -> str:
        """Access the ``slot[slot].isolated.endchannel`` attribute.

        Description:
            - This attribute indicates the last channel in the specified slot that has isolated
              channels.

        Usage:
            - Accessing this property will send the ``print(slot[slot].isolated.endchannel)`` query.

        TSP Syntax:
            ```
            - print(slot[slot].isolated.endchannel)
            ```

        Info:
            - ``slot``, the slot number.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".endchannel"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.endchannel)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.endchannel`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def startchannel(self) -> str:
        """Access the ``slot[slot].isolated.startchannel`` attribute.

        Description:
            - This attribute indicates the first channel in the specified slot that has isolated
              channels.

        Usage:
            - Accessing this property will send the ``print(slot[slot].isolated.startchannel)``
              query.

        TSP Syntax:
            ```
            - print(slot[slot].isolated.startchannel)
            ```

        Info:
            - ``slot``, the slot number.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".startchannel"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.startchannel)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.startchannel`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error


class SlotItemDigitalio(BaseTSPCmd):
    """The ``slot[slot].digitalio`` command tree.

    Info:
        - ``slot``, the slot number.

    Properties and methods:
        - ``.endchannel``: The ``slot[slot].digitalio.endchannel`` attribute.
        - ``.startchannel``: The ``slot[slot].digitalio.startchannel`` attribute.
    """

    @property
    def endchannel(self) -> str:
        """Access the ``slot[slot].digitalio.endchannel`` attribute.

        Description:
            - This attribute indicates the last channel in the specified slot that supports digital
              inputs and outputs.

        Usage:
            - Accessing this property will send the ``print(slot[slot].digitalio.endchannel)``
              query.

        TSP Syntax:
            ```
            - print(slot[slot].digitalio.endchannel)
            ```

        Info:
            - ``slot``, the slot number.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".endchannel"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.endchannel)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.endchannel`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def startchannel(self) -> str:
        """Access the ``slot[slot].digitalio.startchannel`` attribute.

        Description:
            - This attribute indicates the first channel in the specified slot that supports digital
              inputs and outputs.

        Usage:
            - Accessing this property will send the ``print(slot[slot].digitalio.startchannel)``
              query.

        TSP Syntax:
            ```
            - print(slot[slot].digitalio.startchannel)
            ```

        Info:
            - ``slot``, the slot number.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".startchannel"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.startchannel)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.startchannel`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error


class SlotItemAnalogoutput(BaseTSPCmd):
    """The ``slot[slot].analogoutput`` command tree.

    Info:
        - ``slot``, the slot number.

    Properties and methods:
        - ``.endchannel``: The ``slot[slot].analogoutput.endchannel`` attribute.
        - ``.startchannel``: The ``slot[slot].analogoutput.startchannel`` attribute.
    """

    @property
    def endchannel(self) -> str:
        """Access the ``slot[slot].analogoutput.endchannel`` attribute.

        Description:
            - This attribute indicates the last channel in the specified slot that supports analog
              outputs.

        Usage:
            - Accessing this property will send the ``print(slot[slot].analogoutput.endchannel)``
              query.

        TSP Syntax:
            ```
            - print(slot[slot].analogoutput.endchannel)
            ```

        Info:
            - ``slot``, the slot number.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".endchannel"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.endchannel)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.endchannel`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def startchannel(self) -> str:
        """Access the ``slot[slot].analogoutput.startchannel`` attribute.

        Description:
            - This attribute indicates the first channel in the specified slot that supports analog
              outputs.

        Usage:
            - Accessing this property will send the ``print(slot[slot].analogoutput.startchannel)``
              query.

        TSP Syntax:
            ```
            - print(slot[slot].analogoutput.startchannel)
            ```

        Info:
            - ``slot``, the slot number.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".startchannel"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.startchannel)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.startchannel`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error


class SlotItemAmps(BaseTSPCmd):
    """The ``slot[slot].amps`` command tree.

    Info:
        - ``slot``, the slot number.

    Properties and methods:
        - ``.endchannel``: The ``slot[slot].amps.endchannel`` attribute.
        - ``.startchannel``: The ``slot[slot].amps.startchannel`` attribute.
    """

    @property
    def endchannel(self) -> str:
        """Access the ``slot[slot].amps.endchannel`` attribute.

        Description:
            - This attribute indicates the last channel in the specified slot that supports amps
              measurements.

        Usage:
            - Accessing this property will send the ``print(slot[slot].amps.endchannel)`` query.

        TSP Syntax:
            ```
            - print(slot[slot].amps.endchannel)
            ```

        Info:
            - ``slot``, the slot number.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".endchannel"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.endchannel)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.endchannel`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def startchannel(self) -> str:
        """Access the ``slot[slot].amps.startchannel`` attribute.

        Description:
            - This attribute indicates the first channel in the specified slot that supports current
              measurements.

        Usage:
            - Accessing this property will send the ``print(slot[slot].amps.startchannel)`` query.

        TSP Syntax:
            ```
            - print(slot[slot].amps.startchannel)
            ```

        Info:
            - ``slot``, the slot number.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".startchannel"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.startchannel)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.startchannel`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error


class SlotItem(ValidatedDynamicNumberCmd, BaseTSPCmd):
    """The ``slot[slot]`` command tree.

    Info:
        - ``slot``, the slot number.

    Properties and methods:
        - ``.amps``: The ``slot[slot].amps`` command tree.
        - ``.analogoutput``: The ``slot[slot].analogoutput`` command tree.
        - ``.commonsideohms``: The ``slot[slot].commonsideohms`` attribute.
        - ``.digitalio``: The ``slot[slot].digitalio`` command tree.
        - ``.idn``: The ``slot[slot].idn`` attribute.
        - ``.isolated``: The ``slot[slot].isolated`` command tree.
        - ``.matrix``: The ``slot[slot].matrix`` command tree.
        - ``.maxvoltage``: The ``slot[slot].maxvoltage`` attribute.
        - ``.pseudocard``: The ``slot[slot].pseudocard`` attribute.
        - ``.tempsensor``: The ``slot[slot].tempsensor`` attribute.
        - ``.totalizer``: The ``slot[slot].totalizer`` command tree.
        - ``.voltage``: The ``slot[slot].voltage`` command tree.
    """

    def __init__(
        self, device: Optional["TSPControl"] = None, cmd_syntax: str = "slot[slot]"
    ) -> None:
        super().__init__(device, cmd_syntax)
        self._amps = SlotItemAmps(device, f"{self._cmd_syntax}.amps")
        self._analogoutput = SlotItemAnalogoutput(device, f"{self._cmd_syntax}.analogoutput")
        self._digitalio = SlotItemDigitalio(device, f"{self._cmd_syntax}.digitalio")
        self._isolated = SlotItemIsolated(device, f"{self._cmd_syntax}.isolated")
        self._matrix = SlotItemMatrix(device, f"{self._cmd_syntax}.matrix")
        self._totalizer = SlotItemTotalizer(device, f"{self._cmd_syntax}.totalizer")
        self._voltage = SlotItemVoltage(device, f"{self._cmd_syntax}.voltage")

    @property
    def amps(self) -> SlotItemAmps:
        """Return the ``slot[slot].amps`` command tree.

        Info:
            - ``slot``, the slot number.

        Sub-properties and sub-methods:
            - ``.endchannel``: The ``slot[slot].amps.endchannel`` attribute.
            - ``.startchannel``: The ``slot[slot].amps.startchannel`` attribute.
        """
        return self._amps

    @property
    def analogoutput(self) -> SlotItemAnalogoutput:
        """Return the ``slot[slot].analogoutput`` command tree.

        Info:
            - ``slot``, the slot number.

        Sub-properties and sub-methods:
            - ``.endchannel``: The ``slot[slot].analogoutput.endchannel`` attribute.
            - ``.startchannel``: The ``slot[slot].analogoutput.startchannel`` attribute.
        """
        return self._analogoutput

    @property
    def commonsideohms(self) -> str:
        """Access the ``slot[slot].commonsideohms`` attribute.

        Description:
            - This attribute indicates whether a card in the specified slot supports commonside
              channels for 4-wire resistance measurements.

        Usage:
            - Accessing this property will send the ``print(slot[slot].commonsideohms)`` query.

        TSP Syntax:
            ```
            - print(slot[slot].commonsideohms)
            ```

        Info:
            - ``slot``, the slot number.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".commonsideohms"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.commonsideohms)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.commonsideohms`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def digitalio(self) -> SlotItemDigitalio:
        """Return the ``slot[slot].digitalio`` command tree.

        Info:
            - ``slot``, the slot number.

        Sub-properties and sub-methods:
            - ``.endchannel``: The ``slot[slot].digitalio.endchannel`` attribute.
            - ``.startchannel``: The ``slot[slot].digitalio.startchannel`` attribute.
        """
        return self._digitalio

    @property
    def idn(self) -> str:
        """Access the ``slot[slot].idn`` attribute.

        Description:
            - This attribute returns a string that contains information about the switching module.

        Usage:
            - Accessing this property will send the ``print(slot[slot].idn)`` query.

        TSP Syntax:
            ```
            - print(slot[slot].idn)
            ```

        Info:
            - ``slot``, the slot number (1 or 2).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".idn"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.idn)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.idn`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def isolated(self) -> SlotItemIsolated:
        """Return the ``slot[slot].isolated`` command tree.

        Info:
            - ``slot``, the slot number.

        Sub-properties and sub-methods:
            - ``.endchannel``: The ``slot[slot].isolated.endchannel`` attribute.
            - ``.startchannel``: The ``slot[slot].isolated.startchannel`` attribute.
        """
        return self._isolated

    @property
    def matrix(self) -> SlotItemMatrix:
        """Return the ``slot[slot].matrix`` command tree.

        Info:
            - ``slot``, the slot number.

        Sub-properties and sub-methods:
            - ``.columns``: The ``slot[slot].matrix.columns`` attribute.
            - ``.rows``: The ``slot[slot].matrix.rows`` attribute.
        """
        return self._matrix

    @property
    def maxvoltage(self) -> str:
        """Access the ``slot[slot].maxvoltage`` attribute.

        Description:
            - This attribute returns the maximum voltage of all channels on a switching module in
              the specified slot.

        Usage:
            - Accessing this property will send the ``print(slot[slot].maxvoltage)`` query.

        TSP Syntax:
            ```
            - print(slot[slot].maxvoltage)
            ```

        Info:
            - ``slot``, the slot number (1 or 2).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".maxvoltage"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.maxvoltage)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.maxvoltage`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def pseudocard(self) -> str:
        """Access the ``slot[slot].pseudocard`` attribute.

        Description:
            - This attribute specifies a pseudocard to implement for the designated slot.

        Usage:
            - Accessing this property will send the ``print(slot[slot].pseudocard)`` query.
            - Setting this property to a value will send the ``slot[slot].pseudocard = value``
              command.

        TSP Syntax:
            ```
            - slot[slot].pseudocard = value
            - print(slot[slot].pseudocard)
            ```

        Info:
            - ``slot``, the slot number (1 or 2).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".pseudocard"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.pseudocard)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.pseudocard`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @pseudocard.setter
    def pseudocard(self, value: Union[str, float]) -> None:
        """Access the ``slot[slot].pseudocard`` attribute.

        Description:
            - This attribute specifies a pseudocard to implement for the designated slot.

        Usage:
            - Accessing this property will send the ``print(slot[slot].pseudocard)`` query.
            - Setting this property to a value will send the ``slot[slot].pseudocard = value``
              command.

        TSP Syntax:
            ```
            - slot[slot].pseudocard = value
            - print(slot[slot].pseudocard)
            ```

        Info:
            - ``slot``, the slot number (1 or 2).

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".pseudocard", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.pseudocard = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.pseudocard`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def tempsensor(self) -> str:
        """Access the ``slot[slot].tempsensor`` attribute.

        Description:
            - This attribute indicates if the module in the specified slot supports temperature
              sensor channels.

        Usage:
            - Accessing this property will send the ``print(slot[slot].tempsensor)`` query.

        TSP Syntax:
            ```
            - print(slot[slot].tempsensor)
            ```

        Info:
            - ``slot``, the slot number.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".tempsensor"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.tempsensor)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.tempsensor`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def totalizer(self) -> SlotItemTotalizer:
        """Return the ``slot[slot].totalizer`` command tree.

        Info:
            - ``slot``, the slot number.

        Sub-properties and sub-methods:
            - ``.endchannel``: The ``slot[slot].totalizer.endchannel`` attribute.
            - ``.startchannel``: The ``slot[slot].totalizer.startchannel`` attribute.
        """
        return self._totalizer

    @property
    def voltage(self) -> SlotItemVoltage:
        """Return the ``slot[slot].voltage`` command tree.

        Info:
            - ``slot``, the slot number.

        Sub-properties and sub-methods:
            - ``.endchannel``: The ``slot[slot].voltage.endchannel`` attribute.
            - ``.startchannel``: The ``slot[slot].voltage.startchannel`` attribute.
        """
        return self._voltage



"""A collection of classes and other helpful objects to use when generating command drivers.

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.
"""

import re
import sys

from collections import defaultdict
from functools import total_ordering
from typing import Any, Callable, DefaultDict, Optional, Type, Union

END_OF_STRING_DIGITS = re.compile(r"([-\d]+)]?$")
MIDDLE_OF_STRING_DIGITS = re.compile(r"([-\d]+)]?")
# TODO: Drop Python 3.8: Once Python 3.8 is no longer supported,
#  the dynamic parent class can be removed
# pylint: disable=unsubscriptable-object,useless-suppression
ParentDefaultDictClass: Type[DefaultDict[Any, Any]] = (
    defaultdict if sys.version_info < (3, 9) else defaultdict[Any, Any]
)


####################################################################################################
# Exceptions
####################################################################################################
class NoDeviceProvidedError(Exception):
    """A custom exception indicating that no device was provided."""


####################################################################################################
# Classes
####################################################################################################
class DefaultDictPassKeyToFactory(ParentDefaultDictClass):
    """A custom defaultdict.

    This custom defaultdict passes the key used to access a missing value into the stored
    default_factory function as the only parameter.
    """

    def __init__(self, default_factory: Callable[[Union[int, str]], Any], **kwargs: Any) -> None:
        """Create an instance of the class.

        Args:
            default_factory: The factory function used to create new values in the dictionary.
            kwargs: The keyword arguments.
        """
        super().__init__(default_factory, **kwargs)  # type: ignore[arg-type]

    def __missing__(self, key: Any) -> Any:
        """Call the ``default_factory()`` function and pass the key as the only parameter.

        Args:
            key: The key that doesn't pair with a value yet.
        """
        if self.default_factory:
            # noinspection PyArgumentList  # pylint: disable=not-callable
            dict.__setitem__(self, key, self.default_factory(key))  # type: ignore[arg-type]
            return self[key]
        return super().__missing__(key)


@total_ordering  # If comparisons are slowing down the code, implementing the rest would speed it up
class BaseCmd:
    """A class defining the base syntax of a command class.

    The syntax is accessible through the ``.cmd_syntax`` property.
    """

    def __init__(self, device: Optional[Any], cmd_syntax: str) -> None:
        """Instantiate the command.

        Args:
            device: The optional device object,
                if ``None`` is passed in then device communication will not be allowed.
            cmd_syntax: The command syntax.
        """
        self._cmd_syntax = cmd_syntax
        self._device = device

    def __eq__(self, other: object) -> bool:
        """Compare equality by comparing the syntax of two commands."""
        if not isinstance(other, BaseCmd):
            return NotImplemented
        return str(self) == str(other)

    def __lt__(self, other: Any) -> bool:
        """Check if the command is less than another command by comparing the syntax."""
        if not isinstance(other, BaseCmd):
            return NotImplemented
        return str(self) < str(other)

    def __hash__(self) -> int:
        """Provide a hash."""
        return hash(str(self))

    def __str__(self) -> str:
        """Return the syntax as the string representation of the instance object."""
        return self._cmd_syntax

    @property
    def cmd_syntax(self) -> str:
        """Return the syntax."""
        return self._cmd_syntax


class ValidatedDynamicNumberCmd(BaseCmd):  # pylint: disable=too-few-public-methods
    """A mixin class that is used to validate the dynamix item number is valid.

    This class checks the dynamic item number at the end of the SCPI syntax (``.cmd_syntax``
    attribute) to determine if it is a valid dynamic item number (greater than or equal to 1).
    """

    def __init__(self, device: Optional[Any], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)

        # Validate the dynamic item number
        if (match := END_OF_STRING_DIGITS.search(self._cmd_syntax)) is None:
            match = MIDDLE_OF_STRING_DIGITS.search(
                self._cmd_syntax.rsplit(":", maxsplit=1)[-1].rsplit(".", maxsplit=1)[-1]
            )
        if (item_number := int(match.group(1))) < 1:  # type: ignore[union-attr]
            msg = (
                f"{item_number=} is not a valid item number, "
                f"the number must be greater than or equal to 1"
            )
            raise ValueError(msg)
